#!/usr/bin/python2.7
# Copyright 2012 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for cpp_generator."""

__author__ = 'ewiseblatt@google.com (Eric Wiseblatt)'

from google.apputils import basetest
from googleapis.codegen import cpp_generator
from googleapis.codegen import data_types
from googleapis.codegen import data_value


class CppApiTest(basetest.TestCase):

  def testToClassName(self):
    """Test creating safe class names from object names."""
    api = cpp_generator.CppApi({
        'name': 'dummy',
        'version': 'v1',
        'resources': {
            'foo': {
                'methods': {
                    'bar': {
                        'id': 'bar',
                    }
                },
                'resources': {
                    'baz': {
                        'methods': {
                            'qux': {
                                'id': 'qux'
                            }
                        }
                    }
                }
            }
        }
    })
    self.assertEquals('DummyService', api._class_name)
    self.assertEquals('dummy', api.values['name'])

    # We'll capitalize class names. Keywords conflicts arent an
    # issue because keywords are all lowercase.
    self.assertEquals('Foo', api.ToClassName('foo', None))
    self.assertEquals('Class', api.ToClassName('class', None))
    self.assertEquals('Default', api.ToClassName('default', None))
    self.assertEquals('Import', api.ToClassName('import', None))
    self.assertEquals('Object', api.ToClassName('object', None))
    self.assertEquals('String', api.ToClassName('string', None))
    self.assertEquals('True', api.ToClassName('true', None))

    self.assertEquals('DummySerializableJson',
                      api.ToClassName('serializableJson', None))

    # Method and Resource decorate the class name accordingly
    self.assertEquals('Service', api.ToClassName('service', api))
    self.assertEquals('DummyService', api.ToClassName('dummy', api))
    self.assertEquals('DummyserviceService',
                      api.ToClassName('dummyservice', api))
    self.assertEquals('DummyService', api.ToClassName('dummyService', api))

    foo = api._resources[0]
    self.assertEquals('DummyResource', api.ToClassName('dummy', foo))
    self.assertEquals('ResourceResource',
                      api.ToClassName('resource', foo))
    self.assertEquals('AresourceResource',
                      api.ToClassName('Aresource', foo))
    self.assertEquals('AResourceResource',
                      api.ToClassName('AResource', foo))

    bar = foo._methods[0]
    self.assertEquals('FooResource_DummyMethod',
                      api.ToClassName('dummy', bar))
    self.assertEquals('FooResource_MethodMethod',
                      api.ToClassName('method', bar))
    self.assertEquals('FooResource_AmethodMethod',
                      api.ToClassName('amethod', bar))
    self.assertEquals('FooResource_AMethodMethod',
                      api.ToClassName('aMethod', bar))

    baz = foo._resources[0]
    self.assertEquals('DummyResource', api.ToClassName('dummy', baz))

    qux = baz._methods[0]
    self.assertEquals('FooResource_BazResource_DummyMethod',
                      api.ToClassName('dummy', qux))



class CppGeneratorTest(basetest.TestCase):
  def testImportsForArray(self):
    """Test if we get the right imports for an array.

    The goal is to see that an array of a primative type which requires an
    import really works.
    """
    gen = cpp_generator.CppGenerator({
        'name': 'dummy',
        'version': 'v1',
        'resources': {},
        'schemas': {
            'Bar': {
                'id': 'Bar',
                'type': 'object',
                'properties': {
                    'p1': {
                        'type': 'array',
                        'items': {
                            'type': 'string',
                            'format': 'uint64'
                            }
                        },
                    'p2': {
                        'type': 'array',
                        'items': {
                            'type': 'array',
                            'items': {
                                'type': 'string',
                                'format': 'date-time'
                                }
                            }
                        }
                    }
                }
            }
        })
    gen.AnnotateApiForLanguage(gen.api)
# TODO(user): 20121010
# This functionality is not yet imported, but keeping this test copied from
# the Java test as a placeholder
#    found_big_integer = False
#    found_date_time = False
#    for schema in gen.api._schemas.values():
#      import_manager = schema.values.get('importManager')
#      for import_list in import_manager.ImportLists():
#        for import_def in import_list:
#          print import_def
#          if import_def.find('BigInteger'):
#            found_big_integer = True
#          if import_def.find('DateTime'):
#            found_date_time = True
#    self.assertTrue(found_big_integer)
#    self.assertTrue(found_date_time)

  def testDefaultPath(self):
    def MakeGen(owner, the_name='fake', package_path=None):
      api_def = {
          'name': the_name,
          'version': 'v1',
          'rootUrl': 'https://test.domain/X%sY' % owner,
          'servicePath': 'fake/v1',
          'domain': 'domain.com',
          'owner': owner,
          }
      if package_path:
        api_def['packagePath'] = package_path
      gen = cpp_generator.CppGenerator(api_def)
      gen.AnnotateApiForLanguage(gen.api)
      return gen

    gen = MakeGen('company')
    self.assertEquals('company/fake_api', gen.api.module.path)

    gen = MakeGen('My Custom App')
    self.assertEquals('mycustomapp/fake_api', gen.api.module.path)

    gen = MakeGen('company', package_path='My/Custom Path')
    self.assertEquals('company/My/CustomPath/fake_api', gen.api.module.path)

    gen = MakeGen('CamelCase')
    self.assertEquals('camelcase/fake_api', gen.api.module.path)

    gen = MakeGen(owner='owner', the_name='mixedCase')
    self.assertEquals('owner/mixedcase_api', gen.api.module.path)


class CppLanguageModelDataValueTest(basetest.TestCase):

  def setUp(self):
    self.language_model = cpp_generator.CppLanguageModel()

  def _CreateDataValue(self, value, val_type):
    def_dict = {
        'className': 'Foo',
        'type': val_type,
        }
    prototype = data_types.DataType(
        def_dict, None, language_model=self.language_model)
    dv = data_value.DataValue(value, prototype)
    return dv

  def testRenderBoolean(self):
    dv = self._CreateDataValue(True, 'boolean')
    render_method = self.language_model._SUPPORTED_TYPES['boolean']
    self.assertEqual('true', render_method(dv))

    dv.SetValue(False)
    self.assertEqual('false', render_method(dv))

  def testRenderInteger(self):
    dv = self._CreateDataValue(42, 'integer')
    render_method = self.language_model._SUPPORTED_TYPES['integer']
    self.assertRaises(ValueError, render_method, dv)

    dv.SetTemplateValue('codeType', 'int16')
    self.assertEqual('42', render_method(dv))
    dv.SetTemplateValue('codeType', 'int32')
    self.assertEqual('42', render_method(dv))
    dv.SetTemplateValue('codeType', 'int64')
    self.assertEqual('42L', render_method(dv))

    dv.SetTemplateValue('codeType', 'uint32')
    self.assertEqual('42', render_method(dv))

    dv = self._CreateDataValue(-1, 'integer')

    # TODO(user): 20121012
    # These are bugs. We're writing the test to document the wrong behavior
    # for now.
    dv.SetTemplateValue('codeType', 'uint32')
    self.assertEqual('-1', render_method(dv))     # -1 isnt unsigned
    dv = self._CreateDataValue(65536, 'integer')
    dv.SetTemplateValue('codeType', 'int16')
    self.assertEqual('65536', render_method(dv))  # 2^16 doesnt fit in an int16


class CppLanguageModelTest(basetest.TestCase):

  def testImports(self):
    gen = cpp_generator.CppGenerator({
        'name': 'dummy',
        'version': 'v1',
        'resources': {},
        'schemas': {
            'Foo': {
                'id': 'Foo',
                'type': 'object',
                'properties': {
                    's': {
                        'type': 'string',
                    }
                }
            },
            'Bar': {
                'id': 'Bar',
                'type': 'object',
                'properties': {
                    'n32': {
                        'type': 'integer',
                        'format': 'int32',
                    },
                    'un64': {
                        'type': 'integer',
                        'format': 'uint64',
                    },
                    'foo': {
                        'type': 'array',
                        'items': {
                            '$ref': 'Foo'
                        }
                    },
                }
            }
        }
    })
    gen.AnnotateApiForLanguage(gen.api)
    foo_schema = gen.api._schemas['Foo']
    self.assertEqual('Foo', foo_schema.values.get('id'))
    foo_import_manager = foo_schema.values.get('importManager')
    self.assertTrue('<string>' in foo_import_manager.platform_imports)

    bar_schema = gen.api._schemas['Bar']
    self.assertEqual('Bar', bar_schema.values.get('id'))

    bar_import_manager = bar_schema.values.get('importManager')
    self.assertTrue('"googleapis/base/integral_types.h"'
                    in bar_import_manager.google_imports)

  def testDocumentation(self):
    """Test if we get the right documentation annotations.
    """
    gen = cpp_generator.CppGenerator(
        discovery={
            'name': 'dummy',
            'version': 'v1',
            'resources': {},
            'schemas': {}
            })

    # No DL case
    self.assertEqual(gen._DLifyDescription('simple text'), 'simple text')

    # No leading text before the <dl>.
    self.assertEqual(
        gen._DLifyDescription(
            'A - description A\nB - description B'),
        '<dl>\n<dt>A\n<dd>description A.\n<dt>B\n<dd>description B.\n</dl>\n')

    # Leading text before the <dl>.
    self.assertEqual(
        gen._DLifyDescription('Text before:'
                              '\nA - description A\nB - description B\n'
                              'Text after.'),
        'Text before:\n<dl>\n<dt>A\n<dd>description A.\n'
        '<dt>B\n<dd>description B.\n</dl>\n'
        'Text after.')

    # No line breaks after last element (short)
    self.assertEqual(
        gen._DLifyDescription('Text before:'
                              '\nA - description A\n- B - description B. '
                              'Text after.'),
        'Text before:\n<dl>\n<dt>A\n<dd>description A.\n'
        '<dt>B\n<dd>description B. Text after.\n</dl>\n')

    # No line breaks after last element (long)
    blurb = (
        'This text is a bit long. At some point it will cross the threshold '
        'where we consider it runoff. That means we will step back and '
        'truncate the last element in the dl list so that it just includes '
        'the first sentence. All the rest of the sentences, such as these, '
        'will appear after we close the dl.')
    self.assertEqual(
        gen._DLifyDescription(
            'Text before:'
            '\nA - description A\n- B - description B. %s' % blurb),
        'Text before:\n<dl>\n<dt>A\n<dd>description A.\n'
        '<dt>B\n<dd>description B.\n</dl>\n %s' % blurb)


  def testVersionPackage(self):
    gen = cpp_generator.CppGenerator(
        discovery={
            'name': 'dummy',
            'version': 'v1',
            'resources': {},
            'schemas': {
                'Bar': {
                    'id': 'Bar',
                    'type': 'object',
                    'properties': {
                        'n32': {
                            'type': 'integer',
                            'format': 'int32',
                            },
                        'un64': {
                            'type': 'integer',
                            'format': 'uint64',
                            },
                        }
                    }
                }
            },
        options={'version_package': True})
    gen.AnnotateApiForLanguage(gen.api)
    self.assertEquals('google/dummy_api_v1', gen.api.module.path)
    self.assertEquals('google_dummy_api_v1', gen.api.module.name)

  def testReservedWords(self):
    gen = cpp_generator.CppGenerator({
        'name': 'dummy',
        'version': 'v1',
        'schemas': {
            'Bar': {
                'id': 'Bar',
                'type': 'object',
                'properties': {
                    'n32': {
                        'type': 'integer',
                        'format': 'int32',
                        },
                    'class': {
                        'type': 'string',
                        },
                    }
                }
            },
        'resources': {
            'foo': {
                'methods': {
                    'bar': {
                        'id': 'foo.bar',
                        'parameters': {
                            'aJavaStyleName': {
                                'location': 'path',
                                'required': True,
                                'type': 'int',
                                },
                            'namespace': {
                                'location': 'path',
                                'required': True,
                                'type': 'int',
                                },
                            }
                        }
                    }
                }
            }
        })
    gen.AnnotateApiForLanguage(gen.api)
    bar_schema = gen.api.SchemaByName('Bar')
    for p in bar_schema.values['properties']:
      wire_name = p.GetTemplateValue('wireName')
      if wire_name == 'class':
        self.assertEquals('class_', p.memberName)
        self.assertEquals('class__', p.parameterName)

    bar_method = gen.api.MethodByName('foo.bar')
    self.assertEquals(2, len(bar_method.required_parameters))
    for p in bar_method.required_parameters:
      wire_name = p.GetTemplateValue('wireName')
      if wire_name == 'aJavaStyleName':
        self.assertEquals('a_java_style_name_', p.memberName)
        self.assertEquals('a_java_style_name', p.parameterName)
      if wire_name == 'namespace':
        self.assertEquals('namespace_', p.memberName)
        self.assertEquals('namespace__', p.parameterName)


if __name__ == '__main__':
  basetest.main()



#!/usr/bin/python2.7
# Copyright 2011 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""PHP library generator.

This module generates a PHP service class from a given Google APIs Discovery
document. The generated PHP code is intended to be used by the Google
APIs PHP Client (http://code.google.com/p/google-api-php-client).

Features:
- Meta classes generated from the schema definition.
- Type hinting in setter methods and api service methods.
- ApiService classes that offer a PHP interface to the API.
- Proper PHPDoc comments for each class and method in the library (FUTURE).
"""

__author__ = 'chirags@google.com (Chirag Shah)'

import collections
import json
import operator

from googleapis.codegen import api
from googleapis.codegen import api_library_generator
from googleapis.codegen import data_types
from googleapis.codegen import language_model
from googleapis.codegen import utilities
from googleapis.codegen.schema import Schema


class PHPGenerator(api_library_generator.ApiLibraryGenerator):
  """The PHP code generator."""

  def __init__(self, discovery, options=None):
    """Create a new PHPGenerator.

    Args:
      discovery: (dict) The discovery document dictionary.
      options: (dict) A dictionary of options to guide the generator's behavior.
    """
    super(PHPGenerator, self).__init__(PHPApi, discovery, 'php',
                                       language_model=PhpLanguageModel(),
                                       options=options)

  def AnnotateResource(self, the_api, resource):
    """Add the discovery dictionary as data to each resource.

    Override default implementation.

    Prepend the resource class name to each sub-resource since PHP doesn't
    support nested classes.

    Args:
      the_api: (Api) The API this Resource belongs to.
      resource: (Resource) The Resource to annotate.
    """
    resource.json = json.dumps(_StripResource(resource.raw))
    # Escape stray quotes since it will be used in a PHP function call.
    resource.json = resource.json.replace('\'', '\\\'').replace('{}', '')
    for method in resource.values['methods']:
      self.AnnotateMethod(the_api, method, resource)

    if not resource.GetTemplateValue('phpPropName'):
      resource.SetTemplateValue('phpPropName', resource.values['wireName'])

    for r in resource.values['resources']:
      r.values['className'] = (resource.values['className'] +
                               r.values['className'])
      namespaced = '_'.join((resource.GetTemplateValue('phpPropName'),
                             r.values['wireName']))
      namespaced = '_'.join(
          (resource.GetTemplateValue('phpPropName'),
           self.language_model.ToMemberName(r.values['wireName'], None)))
      r.SetTemplateValue('phpPropName', namespaced)
      self.AnnotateResource(the_api, r)

  def AnnotateMethod(self, unused_api, method, resource=None):
    """Format service request method parameters.

    Override default implementation.

    Annotates each method parameter with a type hint if possible.
    Adds the postBody parameter if there's a requestType parameter.
    Generates a list of service parameters used by the client library.

    Args:
      unused_api: (Api) The API this Method belongs to.
      method: (Method) The Method to annotate.
      resource: (Resource) The Resource which owns this Method.
    """
    for param in method.parameters:
      self._SetTypeHint(param)

    if method.parameters or method.values.get('requestType'):
      method.SetTemplateValue('hasParams', True)
    method.SetTemplateValue('name', self._ToMethodName(method.values, resource))

  def AnnotateProperty(self, unused_api, prop, schema):
    """Annotate properties with a PHP type hint.

    Overrides default implementation.

    Args:
      unused_api: (Api) The API this Property belongs to.
      prop: (Property) The Property to annotate.
      schema: (Schema) The Schema this Property belongs to.
    """
    if isinstance(prop.data_type, data_types.ArrayDataType):
      prop.SetTemplateValue('dataType', 'array')
      schema.SetTemplateValue('dataType', 'array')
      collection = self.language_model.ToMemberName(prop['wireName'], 0)
      schema.SetTemplateValue('collectionKey', collection)
    elif not schema.GetTemplateValue('dataType'):
      schema.SetTemplateValue('dataType', 'object')

    if isinstance(prop.data_type, data_types.MapDataType):
      prop.SetTemplateValue('dataType', 'map')

    if not prop.member_name_is_json_name:
      schema.SetTemplateValue('has_gapi', True)

    #  add the prop name
    prop_names = schema.values.get('propNames', [])
    prop_names.append(prop.memberName)
    schema.SetTemplateValue('propNames', prop_names)

    self._SetTypeHint(prop)

  def _GenerateLibrarySource(self, the_api, source_package_writer):
    """Default operations to generate the package.

    Do all the default operations for generating a package.
    1. Walk the template tree to generate the source.
    2. Add in per-language additions to the source
    3. Optionally copy in dependencies
    4. (Side effect) Closes the source_package_writer.

    Args:
      the_api: (Api) The Api instance we are writing a libary for.
      source_package_writer: (LibraryPackage) source output package.
    """
    list_replacements = {
        '___models_': ['model', the_api.ModelClasses()],
        '___resources_': ['resource', the_api.ResourceClasses()],
        '___topLevelModels_': ['model', the_api.TopLevelModelClasses()],
        }
    self.WalkTemplateTree('templates', self._path_replacements,
                          list_replacements,
                          self._top_level_defines, source_package_writer)
    # Call back to the language specific generator to give it a chance to emit
    # special case elements.
    self.GenerateExtraSourceOutput(source_package_writer)

  def _ToMethodName(self, method, resource):
    """Convert a wire format name into a suitable PHP variable name."""
    s = method['wireName']
    if resource and (s.lower() in PhpLanguageModel.PHP_KEYWORDS):
      s += resource['className']
    # Ensure dashes don't show up in method names
    words = s.split('-')
    return ''.join(words[:1] + [w.capitalize() for w in words[1:]])

  def _SetTypeHint(self, prop):
    """Strip primitive types since PHP doesn't support primitive type hints."""
    code_type = prop.code_type
    if code_type and code_type.lower() in PhpLanguageModel.PHP_TYPES:
      prop.values['typeHint'] = ''
      prop.values['typeHintOld'] = ''
    else:
      prop.values['typeHintOld'] = ('%s_%s' %
                                    (self._api.values['owner'].title(),
                                     code_type))
      prop.values['typeHint'] = ('%s_Service_%s_%s' %
                                 (self._api.values['owner'].title(),
                                  self._api.values['className'], code_type))


class PhpLanguageModel(language_model.LanguageModel):
  """A LanguageModel tunded for PHP."""

  language = 'php'

  _SCHEMA_TYPE_TO_PHP_TYPE = {
      'any': 'array',
      'boolean': 'bool',
      'integer': 'int',
      'long': 'string',  # PHP doesn't support long integers.
      'number': 'double',
      'string': 'string',
      'uint32': 'string',  # PHP doesn't support unsigned integers.
      'uint64': 'string',  # PHP doesn't support unsigned integers.
      'int32': 'int',
      'int64': 'string',  # Size of an integer is platform-dependent.
      'double': 'double',
      'float': 'float',
      }

  # Populated from http://php.net/manual/en/reserved.keywords.php
  PHP_KEYWORDS = frozenset((
      'abstract', 'and', 'array', 'as', 'break', 'call', 'callable',
      'case', 'catch', 'cfunction', 'class', 'clone',
      'const', 'continue', 'declare', 'default', 'do',
      'else', 'elseif', 'empty', 'enddeclare', 'endfor', 'endforeach',
      'endif', 'endswitch', 'endwhile', 'extends', 'final',
      'finally', 'for', 'foreach', 'function', 'global', 'goto',
      'if', 'implements', 'interface', 'instanceof', 'list',
      'namespace', 'new', 'old_function', 'or', 'parent', 'private',
      'protected', 'public', 'return', 'static', 'switch', 'throw',
      'trait', 'try', 'unset', 'use', 'var', 'while', 'xor', 'yield',
      ))

  PHP_TYPES = frozenset((
      'bool', 'boolean', 'int', 'integer', 'file', 'float', 'double', 'string',
      'array', 'object',
      'null', 'resource',
      ))

  # We can not create classes which match a PHP keyword or built in object type.
  RESERVED_CLASS_NAMES = PHP_KEYWORDS | PHP_TYPES

  array_of_policy = language_model.NamingPolicy(format_string='{name}')
  map_of_policy = language_model.NamingPolicy(format_string='{name}')
  member_policy = language_model.NamingPolicy(
      case_transform=language_model.LOWER_CAMEL_CASE)
  getter_policy = language_model.NamingPolicy(
      case_transform=language_model.UPPER_CAMEL_CASE,
      format_string='get{name}')
  setter_policy = language_model.NamingPolicy(
      case_transform=language_model.UPPER_CAMEL_CASE,
      format_string='set{name}')

  def __init__(self):
    super(PhpLanguageModel, self).__init__(class_name_delimiter='.')

  def GetCodeTypeFromDictionary(self, def_dict):
    """Convert a json primitive type to a suitable PHP type name.

    Overrides the default.

    Args:
      def_dict: (dict) A dictionary describing Json schema for this Property.
    Returns:
      A name suitable for use as a class in PHP.
    """

    # Unique identifier for this schema.
    json_type = def_dict.get('type', 'string')

    # An additional regular expression or key that helps constrain the value.
    # See: http://tools.ietf.org/html/draft-zyp-json-schema-03#section-5.23
    json_format = def_dict.get('format')

    # If we know what to do with this format type, then use native type.
    php_type = (self._SCHEMA_TYPE_TO_PHP_TYPE.get(json_format)
                or self._SCHEMA_TYPE_TO_PHP_TYPE.get(json_type, json_type))
    return php_type

  def ToMemberName(self, s, unused_api):
    """Convert a wire format name into a suitable PHP variable name.

    Overrides the default.

    Args:
      s: (string) The wire format name of a member variable.
    Returns:
      A name suitable for use as a member in PHP.
    """
    return s.replace('-', '_').replace('.', '_').replace('/', '__')


class PHPApi(api.Api):
  """An Api with PHP annotations."""

  def __init__(self, discovery_doc, language=None):
    super(PHPApi, self).__init__(discovery_doc, language)
    # We've been advised by the open source office that the correct legal thing
    # to do is to hard-code a copyright date. Generated clients should always
    # display the year 2014, even if they are generated in subsequent years.
    self.SetTemplateValue('copyright', 'Copyright 2014 Google Inc.\n')

  # pylint: disable=unused-argument
  # The parameter element_type is deliberately unused since PHP doesn't
  # support nested classes.
  def ToClassName(self, s, unused_element, element_type=None):
    """Convert a discovery name to a suitable PHP class name.

    Overrides the default.

    Args:
      s: (string) The wire format name of a class.
      unused_element: (object) The object we need a class name for.
      element_type: (string) The kind of object we need a class name for.
    Returns:
      A name suitable for use as a class in PHP.
    """
    if s.lower() in PhpLanguageModel.RESERVED_CLASS_NAMES:
      # Prepend the service name.
      return utilities.CamelCase(self.values['name']) + utilities.CamelCase(s)
    return utilities.CamelCase(s)

  def ModelClasses(self):
    """Return all the model classes."""
    ret = set(
        s for s in self._schemas.itervalues()
        if isinstance(s, Schema))
    return sorted(ret, key=operator.attrgetter('class_name'))

  def ResourceClasses(self, resources=None):
    """Return all the resource classes."""
    if resources is None:
      resources = self.values['resources']

    all_resources = sorted(resources,
                           key=lambda resource: resource.values['className'])

    for resource in resources:
      all_resources.extend(self.ResourceClasses(resource.values['resources']))

    return all_resources

  def _BuildResourceDefinitions(self):
    """Loop over the resources in the discovery doc and build definitions."""
    self._resources = []
    def_dict = self.values.get('resources') or {}
    for name in sorted(def_dict):
      method_dict = def_dict[name].get('methods', {})
      for n in method_dict:
        # make sure all parameters are of type dict (and not list)
        if not method_dict[n].get('parameters'):
          method_dict[n]['parameters'] = {}
      resource = api.Resource(self, name, def_dict[name], parent=self)
      self._resources.append(resource)

# Properties that should be stripped when serializing parts of the
# discovery document.
_EXTRA_PROPERTIES = ['description', 'enumDescriptions', 'resources', 'pattern',
                     'parameterOrder']


def _StripResource(resource):
  """Return a copy of a resource dict with extraneous properties removed."""
  if not isinstance(resource, dict):
    return resource
  ret = collections.OrderedDict()
  for name, value in resource.iteritems():
    if name not in _EXTRA_PROPERTIES:
      ret[name] = _StripResource(value)
  return ret



#!/usr/bin/python2.7
# Copyright 2011 Google Inc. All Rights Reserved.

"""Targets class describes which languages/platforms we support."""

__author__ = 'wclarkso@google.com (Will Clarkson)'

import logging
import os


from googleapis.codegen.filesys import files
from googleapis.codegen.utilities import json_expander
from googleapis.codegen.utilities import json_with_comments


class Targets(object):
  """Targets maintains the list of possible target options.

  Reads targets.json file in local directory. This file is formatted
  as:
  {
  'languages': {
    'languageA': {
      'surface_option1': {
        'path': 'stable',
        'description': 'something about language A',
        'displayName': 'SurfaceOption1',
      },
      'surface_option2': {
        'path': 'experimental',
        'description': 'something about language A',
        'displayName': 'SurfaceOption2',
        'platforms': ['cmd-line'],
      }
     },
    'languageB': {
      ...
    }, ...
    },
  'platforms': {
    'cmd-line': {
      'displayName': 'Pretty Platform Name'
    }
   }
  }
  """

  def __init__(self, targets_path=None, template_root=None, targets_dict=None):
    """Constructor.

    Loads targets file.

    Args:
      targets_path: (str) Path to targets file. Defaults to './targets.json'
      template_root: (str) Path to template root. Defaults to '.'
      targets_dict: (dict) Initial data, if not supplied from a file.

    Raises:
      ValueError: if the targets file does not contain the required sections.
    """
    self.template_root = template_root or Targets._default_template_root
    self.targets_path = targets_path or os.path.join(self.template_root,
                                                     'targets.json')
    if targets_dict:
      self._targets_dict = targets_dict
    else:
      self._targets_dict = json_with_comments.Loads(
          files.GetFileContents(self.targets_path))

    # Do some basic validation that this has the required fields
    if 'languages' not in self._targets_dict:
      raise ValueError('languages not in targets.json')

  def Dict(self):
    """The targets.json file as a dictionary."""
    return self._targets_dict

  def VariationsForLanguage(self, language):
    language_def = self._targets_dict['languages'].get(language)
    if not language_def:
      return None
    return Variations(self, language, language_def['variations'])

  def GetLanguage(self, language):
    return self._targets_dict['languages'][language]

  def Languages(self):
    return self._targets_dict['languages']

  def Platforms(self):
    return self._targets_dict.get('platforms', {})

  @staticmethod
  def SetDefaultTemplateRoot(path):
    """Sets a new default full path to the templates directory.

    Args:
      path: (str) full path to templates directory.
    """
    # This is not a classmethod because we don't want subclasses
    # to shadow this value.
    logging.info('setting default template root to %s', path)
    Targets._default_template_root = path

  @staticmethod
  def GetDefaultTemplateRoot():
    return Targets._default_template_root

  # Set the initial template root.
  _default_template_root = os.path.join(os.path.dirname(__file__),
                                        'languages')

  # Whether to use variation release versions when calculating template paths.
  use_versioned_paths = False

  @staticmethod
  def SetUseVersionedPaths(use_versioned_paths):
    """Sets whether versions are used in the template path."""
    # This is not a classmethod because we don't want subclasses
    # to shadow this value.
    Targets.use_versioned_paths = use_versioned_paths


class Variations(dict):
  """A set of variations available for a particular language."""

  def __init__(self, targets, language, variations_dict):
    super(Variations, self).__init__(variations_dict)
    self._targets = targets
    self._language = language

  def IsValid(self, variation):
    """Test is a variation exists."""
    return variation in self

  def _RelativeTemplateDir(self, variation):
    """Returns the path to template dir for the selected variation.

    By default, the path is the same as the variation name. It can be
    overridden in two ways, of descending precedence:
      1. by the 'releaseVersion' element, if use_versioned_paths is set.
      2. with an explicit 'path' statement.

    Args:
      variation: (str) A target variation name.
    Returns:
      (str) Relative path to template directory.
    """
    if self._targets.use_versioned_paths:
      path = self[variation].get('releaseVersion') or variation
    else:
      path = None
    if not path:
      path = self.get(variation, {}).get('path') or variation
    return os.path.join(self._language, path)

  def AbsoluteTemplateDir(self, variation):
    """Returns the path to template dir for the selected variation.

    Args:
      variation: (str) A target variation name.
    Returns:
      (str) Absolute path to template directory.
    """
    return os.path.join(self._targets.template_root,
                        self._RelativeTemplateDir(variation))

  def GetFeaturesForReleaseVersion(self, release_version):
    for name in self:
      features = self.GetFeatures(name)
      if release_version == features.get('releaseVersion'):
        return features
    return None

  def GetFeatures(self, variation):
    """Returns the features dictionary for a specific variation.

    This is the basic dictionary informaion plus any specific overrides in
    the per-template-tree features.json file.

    Args:
      variation: (str) A target variation name.
    Returns:
      (Features) features dictionary
    """
    if not variation:
      return None
    template_dir = self.AbsoluteTemplateDir(variation)
    features = Features(template_dir, self.get(variation), variation)
    json_path = os.path.join(template_dir, 'features.json')

    try:
      features_json = files.GetFileContents(json_path)
    except files.FileDoesNotExist:
      # for backwards compatibility, we forgive this.
      # TODO(user): be stricter about this and
      # fix/remove any tests that fail as a result.
      return features

    features.update(json_expander.ExpandJsonTemplate(
        json_with_comments.Loads(features_json)))
    # If not specified, the releaseVersion matches the variation
    if not features.get('releaseVersion'):
      features['releaseVersion'] = variation
    return features


class Features(dict):
  """A dictionary describing the features of a particular API variation."""

  # TODO(user): Do we need initial_content? The only thing we see in it is
  # path, which should be set explicitly to the dirname of the real file path.
  def __init__(self, template_dir, initial_content=None, name=None):
    super(Features, self).__init__(initial_content or {})
    self.name = name
    self.template_dir = template_dir
    if 'path' not in self:
      self['path'] = os.path.basename(template_dir)

  def DependenciesForEnvironment(self, environment=None):
    """Returns the list of dependencies for an environment.

    Given an environment:
       build the list of dependencies required for that environment. This
         includes elements marked as all (platform='*') and ones specifically
         mentioning that environment.
       build the list of optional packages which might be useful to your app.
         That is, everything marked generic, but not in the first list.
       build the list of everything excluded from the first two sets

    Args:
      environment: (str) An environment (as per platforms.PLATFORMS). If None,
          the optional packages list will include everything that is not
          mandatory (i.e. marked with platform='*').
    Returns:
      list(dict), list(dict), list(dict): required_packages, optional_packages,
          packages we do not want.
    """
    required = []
    optional = []
    excluded = []
    for r in self.get('requires', []):
      environments = r['environments']
      if '*' in environments or environment in environments:
        required.append(r)
      elif 'generic' in environments or not environment:
        optional.append(r)
      else:
        excluded.append(r)
    return required, optional, excluded

  def ExtractPathsFromDependencies(self, dependencies, file_type=None):
    """Extract the file paths from a list of dependencies.

    Args:
      dependencies: (list(str)) list of dependencies from a Features object
      file_type: (str) If specified, only extract paths for that file type.
    Returns:
      set(str): The set of file paths required for this dependency set.
    """
    ret = set()
    for d in dependencies or []:
      for f in d.get('files') or []:
        p = f.get('path')
        if p and (file_type is None or file_type == f.get('type')):
          ret.add(p)
    return ret

  def AllDependencyPaths(self):
    """Returns the set of all file paths mentioned as dependencies.

    Returns:
      set(str)
    """
    ret = set()
    for dependency in self.get('requires', []):
      for f in dependency.get('files') or []:
        p = f.get('path')
        if p:
          ret.add(p)
    return ret

  def FilePathsWeDoNotDependOn(self, environment=None, file_type=None):
    """Returns the list of file paths which are NOT required for an environment.

    Figure out the files we need for an environment and reduce that by the
    kind of files (if we only want source or binary), then invert that list
    w.r.t. all the files mentioned in the features requirements list.

    The rationale for this function is to make it easy to find the set of
    files which should be stripped from a download, while leaving all files
    not explicitly mentioned in the features.

    Args:
      environment: (str) An environment (as per platforms.PLATFORMS). If None,
          return have the optional packages list include everything that is
          not requried.
      file_type: (str) If specified, only extract paths for that file type.
    Returns:
      list(str): The paths which are NOT required for that platform
    """
    if not environment and not file_type:  # quick exit for common case
      return []
    req, _, _ = self.DependenciesForEnvironment(environment=environment)
    req_paths = self.ExtractPathsFromDependencies(req, file_type=file_type)
    all_paths = self.AllDependencyPaths()
    return all_paths - req_paths



#!/usr/bin/python2.7
# Copyright 2011 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""C# library generator.

This module generates C# code from a Google API discovery documents.
"""

__author__ = 'aiuto@google.com (Tony Aiuto)'

from googleapis.codegen import api
from googleapis.codegen import api_library_generator
from googleapis.codegen import language_model
from googleapis.codegen import schema
from googleapis.codegen import utilities


class CSharpLanguageModel(language_model.LanguageModel):
  """A LanguageModel for C#."""

  language = 'csharp'

  # Allowed '<>' for the usage of Nullable<xxx>
  # Allowed ',' to enable it in a dictionary
  allowed_characters = '<>,'

  # Set up variable naming policy
  array_of_policy = language_model.NamingPolicy(
      format_string='System.Collections.Generic.IList<{name}>')
  map_of_policy = language_model.NamingPolicy(
      format_string='System.Collections.Generic.IDictionary<string,{name}>')
  constant_policy = language_model.NamingPolicy(
      case_transform=language_model.UPPER_CAMEL_CASE)
  class_name_policy = language_model.NamingPolicy(
      case_transform=language_model.UPPER_CAMEL_CASE, separator='.')
  member_policy = language_model.NamingPolicy(
      case_transform=language_model.UPPER_CAMEL_CASE)

  # TODO(user): Fix these 3 tables
  _SCHEMA_TYPE_TO_CSHARP_TYPE = {
      ('any', None): 'object',
      ('boolean', None): 'System.Nullable<bool>',
      ('integer', None): 'System.Nullable<int>',
      ('number', None): 'System.Nullable<double>',
      ('integer', 'int16'): 'System.Nullable<short>',
      ('integer', 'int32'): 'System.Nullable<int>',
      ('integer', 'uint32'): 'System.Nullable<long>',
      ('number', 'double'): 'System.Nullable<double>',
      ('number', 'float'): 'System.Nullable<float>',
      ('string', None): 'string',
      ('string', 'byte'): 'string',
      ('string', 'date'): 'string',  # Date only in 'yyyy-mm-dd' format
      ('string', 'date-time'): 'System.Nullable<System.DateTime>',
      ('string', 'int64'): 'System.Nullable<long>',
      ('string', 'uint64'): 'System.Nullable<ulong>',
      ('object', None): 'object',
      }

  _CSHARP_KEYWORDS = [
      'abstract', 'as', 'base', 'bool', 'break', 'byte', 'case', 'catch',
      'char', 'checked', 'class', 'const', 'continue', 'decimal', 'default',
      'delegate', 'do', 'double', 'else', 'enum', 'event', 'explicit', 'extern',
      'false', 'finally', 'fixed', 'float', 'for', 'foreach', 'goto', 'if',
      'implicit', 'in', 'int', 'interface', 'internal', 'is', 'lock', 'long',
      'namespace', 'new', 'null', 'object', 'operator', 'out', 'override',
      'params', 'private', 'protected', 'public', 'readonly', 'ref', 'return',
      'sbyte', 'sealed', 'short', 'sizeof', 'stackalloc', 'static', 'string',
      'struct', 'switch', 'this', 'throw', 'true', 'try', 'typeof', 'uint',
      'ulong', 'unchecked', 'unsafe', 'ushort', 'using', 'virtual', 'void',
      'volatile', 'while', 'async', 'await'
      ]

  RESERVED_CLASS_NAMES = _CSHARP_KEYWORDS
  reserved_words = RESERVED_CLASS_NAMES

  def __init__(self):
    super(CSharpLanguageModel, self).__init__(class_name_delimiter='.')

  def GetCodeTypeFromDictionary(self, def_dict):
    """Gets an element's data type from its JSON definition.

    Overrides the default.

    Args:
      def_dict: (dict) The defintion dictionary for this type
    Returns:
      A name suitable for use as a C# data type
    """
    json_type = def_dict.get('type', 'string')
    json_format = def_dict.get('format')
    native_type = self._SCHEMA_TYPE_TO_CSHARP_TYPE.get((json_type, json_format))
    if not native_type:
      native_type = 'object'
    return native_type

  def ApplyPolicy(self, policy_name, variable, value):
    """Override the default policy applier so we can do special things."""

    # We need to look for class members clashing with the name of the container.
    # TODO(user): This begs for having a more extensible conflict policy
    # mechanism in LanguageModel.
    if policy_name == 'member':
      return self.ToClassMemberName(variable, value)
    return super(CSharpLanguageModel, self).ApplyPolicy(policy_name, variable,
                                                        value)

  def ToClassMemberName(self, variable, name):
    formatted = super(CSharpLanguageModel, self).ApplyPolicy('member', variable,
                                                             name)
    if isinstance(variable, schema.Property):
      if variable.schema.values.get('className') == formatted:
        formatted += 'Value'
    return formatted

  def ToMemberName(self, s, the_api):
    """CamelCase a wire format name into a suitable C# variable name."""
    camel_s = utilities.CamelCase(s)
    if s in self.RESERVED_CLASS_NAMES:
      # Prepend the service name
      return '%s%s' % (the_api.values['name'], camel_s)
    return camel_s[0].lower() + camel_s[1:]

  def DefaultContainerPathForOwner(self, module):
    """Returns the default path for the module containing this API."""
    return '%s/Apis' % utilities.CamelCase(module.owner_name)

  def ToSafeClassName(self, name, the_api, parent):  # pylint:disable=unused-argument
    # Retrieve the object in question.
    if parent:
      for child in parent.children:
        if child.get('wireName') == name:
          return child.className.split('.')[-1]
    # Fall through if it cannot be found
    if parent:
      name += '_data'
    return utilities.CamelCase(name)

CSHARP_LANGUAGE_MODEL = CSharpLanguageModel()


class CSharpGenerator(api_library_generator.ApiLibraryGenerator):
  """The C# code generator."""

  _NULLABLE_TYPE_TO_REQUIRED = {
      'object': 'object',
      'string': 'string',
      'System.Nullable<bool>': 'bool',
      'System.Nullable<short>': 'short',
      'System.Nullable<int>': 'int',
      'System.Nullable<long>': 'long',
      'System.Nullable<ulong>': 'ulong',
      'System.Nullable<double>': 'double',
      'System.Nullable<float>': 'float',
      'System.Nullable<System.DateTime>': 'System.DateTime',
      }

  def __init__(self, discovery, options=None):
    options = options or {}
    options['version_module'] = True
    # Make sure package_path is capitalized.  We pass it in through options
    # to avoid modifying discovery in place.
    #
    # TODO(user): Refactor so the language can affect the entire module
    # path at once.
    package_path = options.get('package_path', discovery.get('packagePath'))
    if package_path:
      options['package_path'] = '.'.join(
          [s[0].upper() + s[1:] for s in package_path.split('.') if s])
    super(CSharpGenerator, self).__init__(
        CSharpApi,
        discovery,
        language='csharp',
        language_model=CSHARP_LANGUAGE_MODEL,
        options=options)

  def AnnotateParameter(self, unused_method, parameter):
    """Annotate a Parameter with C# specific elements."""
    self._Annotate(parameter)

  def AnnotateProperty(self, unused_api, prop, unused_schema):
    """Annotate a Property with C# specific elements."""
    self._Annotate(prop)

  def _Annotate(self, element):
    """Handles imports for the specified element.

    Args:
      element: (Property|Parameter) The property we will set it's nullable data
               type.
    """
    element.SetTemplateValue('requiredType', self._NULLABLE_TYPE_TO_REQUIRED.
                             get(element.codeType))

  def AnnotateApi(self, the_api):
    """Annotate a Api with C# specific elements."""
    the_api.values['revision'] = self._GetValidRevision(
        the_api.values['revision'])
    super(CSharpGenerator, self).AnnotateApi(the_api)

  def _GetValidRevision(self, rev):
    """Get a valid revision (non negative numeric and less or equal than 65535).

    Args:
      rev: revision number
    Returns:
      A valid revision number.
    """

    try:
      valid_rev = int(rev)
      if valid_rev > 65535 or valid_rev < 0:
        return 0
      return valid_rev
    except ValueError:
      return 0


class CSharpApi(api.Api):
  """An Api with C# annotations."""

  def __init__(self, discovery_doc, **unused_kwargs):
    super(CSharpApi, self).__init__(discovery_doc)
    self.module.SetPath(
        '%s/%s' % (self.values['className'],
                   self.values['versionNoDots'].replace('-', '')))
    self.model_module.SetPath('Data')

  def NestedClassNameForProperty(self, name, owning_schema):
    parent_name = owning_schema.class_name.split('.')[-1]
    class_name = utilities.CamelCase(name) + 'Data'
    if parent_name == class_name:
      class_name += 'Schema'
    return '%s.%s' % (parent_name, class_name)

  # pylint: disable=unused-argument
  def ToClassName(self, s, element, element_type=None):
    """Convert a discovery name to a suitable C# class name.

    Overrides the default.

    Args:
      s: (str) A rosy name of data element.
      element: (object) The object we need a class name for.
      element_type: (str) The kind of element (resource|method) to name.
    Returns:
      A name suitable for use as a class in the generator's target language.
    """
    if s in CSharpLanguageModel.RESERVED_CLASS_NAMES:
      # Prepend the service name
      return '%s%s' % (utilities.CamelCase(self.values['name']),
                       utilities.CamelCase(s))
    return '.'.join([utilities.CamelCase(x) for x in s.split('.')])



#!/usr/bin/python2.7
# Copyright 2010 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for api.py."""

__author__ = 'aiuto@google.com (Tony Aiuto)'

import json
import os


import gflags as flags
from google.apputils import basetest

from googleapis.codegen import data_types
from googleapis.codegen import language_model
from googleapis.codegen.api import Api
from googleapis.codegen.api import AuthScope
from googleapis.codegen.api import Method
from googleapis.codegen.api import Resource
from googleapis.codegen.api import Schema
from googleapis.codegen.api_exception import ApiException

FLAGS = flags.FLAGS



class FakeLanguageModel(language_model.LanguageModel):

  def GetCodeTypeFromDictionary(self, def_dict):
    return def_dict.get('type')

  def ArrayOf(self, unused_var, s):
    return 'Array[%s]' % s


class ApiTest(basetest.TestCase):

  # The base discovery doc for most tests.
  _TEST_DISCOVERY_DOC = 'sample_discovery.json'
  _TEST_DISCOVERY_RPC_DOC = 'sample_discovery.rpc.json'
  _TEST_SHARED_TYPES_DOC = 'sample_shared.json'

  def ApiFromDiscoveryDoc(self, path):
    """Load a discovery doc from a file and creates a library Api.

    Args:
      path: (str) The path to the document.

    Returns:
      An Api for that document.
    """

    f = open(os.path.join(os.path.dirname(__file__), 'testdata', path))
    discovery_doc = json.loads(f.read())
    f.close()
    return Api(discovery_doc)

  def testLazySchemaForCreation(self):
    """Check loading schemas which are known to have a forward reference.

    In the test data, "Activity" refers to "Commment", and the nature
    (sorted) of the loading code causes "Activity" to be processed
    before "Commment".  We want to make sure that SchemaFor does the right
    thing with the lazy creation of activity.
    """
    api = self.ApiFromDiscoveryDoc(self._TEST_DISCOVERY_DOC)
    for schema in ['Activity', 'Comment', 'Activity.object']:
      self.assertTrue(isinstance(api._schemas[schema], Schema))

  def SchemaRefInProperties(self):
    """Make sure that an object ref works in a schema properties list."""
    api = self.ApiFromDiscoveryDoc(self._TEST_DISCOVERY_DOC)
    activity_schema = api._schemas['Activity']
    for prop in activity_schema.values['properties']:
      if prop.values['wireName'] == 'object':
        self.assertEquals('ActivityObject',
                          prop.object_type.values['className'])

  def testMakeDefaultSchemaNameFromTheDictTag(self):
    """Use the outer tag as id for schemas which have no id in their dict."""
    discovery_doc = json.loads(
        """
        {
         "name": "fake",
         "version": "v1",
         "schemas": {
           "should_use_id": {
             "id": "named",
             "type": "object",
             "properties": { "dummy": { "type": "string" } }
           },
           "unnamed": {
             "type": "object",
             "properties": { "dummy": { "type": "string" } }
           }
         },
         "resources": {}
        }
        """)
    gen = Api(discovery_doc)
    self.assertTrue('named' in gen._schemas)
    self.assertTrue('unnamed' in gen._schemas)

  def testUnknownHttpMethod(self):
    """Make sure we get an exception on unknown HTTP types."""
    api = Api({'name': 'dummy', 'version': 'v1', 'resources': {}})
    unused_resource = Resource(api, 'temp', {'methods': {}})
    self.assertRaises(ApiException,
                      Method, api, 'bad', {
                          'rpcMethod': 'rpc',
                          'httpMethod': 'Not GET/POST/PUT/DELETE',
                          'parameters': {}
                      })

  def testRequiredParameterList(self):
    """Make sure we are computing required parameters correctly."""
    api = self.ApiFromDiscoveryDoc(self._TEST_DISCOVERY_DOC)

    tests_executed = 0

    for resource in api.values['resources']:
      if resource.values['wireName'] == 'activities':
        for method in resource.values['methods']:
          if method.required_parameters:
            required_names = [p.values['wireName']
                              for p in method.required_parameters]
            self.assertEquals(method.values['parameterOrder'], required_names)
            tests_executed += 1

    method = api.MethodByName('chili.activities.get')
    optional_names = set(p.values['wireName']
                         for p in method.optional_parameters)
    self.assertEquals(set(['truncateAtom', 'max-comments', 'hl', 'max-liked']),
                      optional_names)
    tests_executed += 1
    self.assertEquals(7, tests_executed)

  def testPageable(self):
    """Make sure pageable methods are identified correctly."""
    api = self.ApiFromDiscoveryDoc(self._TEST_DISCOVERY_DOC)

    # non-pageable method
    count = api.MethodByName('chili.activities.count')
    self.assertIsNone(count.values.get('isPageable'))
    self.assertIsNone(count.values.get('isPagingStyleStandard'))

    # non-pageable method with request page token
    update = api.MethodByName('chili.activities.update')
    self.assertIsNone(update.values.get('isPageable'))
    self.assertIsNone(update.values.get('isPagingStyleStandard'))

    # non-pageable method with response page token
    list_related = api.MethodByName('chili.related.list')
    self.assertIsNone(list_related.values.get('isPageable'))
    self.assertIsNone(list_related.values.get('isPagingStyleStandard'))

    # pageable method with common page token names
    list_activities = api.MethodByName('chili.activities.list')
    self.assertEquals(list_activities.values.get('isPageable'), True)
    self.assertEquals(list_activities.values.get('isPagingStyleStandard'),
                      True)

    # pageable method with uncommon page token names
    list_by_album = api.MethodByName('chili.photos.listByAlbum')
    self.assertEquals(list_by_album.values.get('isPageable'), True)
    self.assertEquals(list_by_album.values.get('isPagingStyleStandard'), False)

    # pageable method with page token in request body
    track = api.MethodByName('chili.activities.track')
    self.assertEquals(track.values.get('isPageable'), True)
    self.assertEquals(track.values.get('isPagingStyleStandard'), False)

  def testSchemaLoadingAsString(self):
    """Test for the "schema as strings" representation."""
    api = self.ApiFromDiscoveryDoc('foo.v1.json')
    self.assertEquals(4, len(api._schemas))

  def testSubResources(self):
    """Test for the APIs with subresources."""

    def CountResourceTree(resource):
      ret = 0
      for r in resource._resources:
        ret += 1 + CountResourceTree(r)
      return ret

    api = self.ApiFromDiscoveryDoc('moderator.v1.json')
    top_level_resources = 0
    total_resources = 0
    non_method_resources = 0
    have_sub_resources = 0
    have_sub_resources_and_methods = 0
    for r in api._resources:
      top_level_resources += 1
      total_resources += 1 + CountResourceTree(r)
      if not r._methods:
        non_method_resources += 1
      if r._resources:
        have_sub_resources += 1
      if r._resources and r._methods:
        have_sub_resources_and_methods += 1
    # Hand counted 18 resources in the file.
    self.assertEquals(18, total_resources)
    self.assertEquals(11, top_level_resources)
    # 4 of them have no methods, only sub resources
    self.assertEquals(4, non_method_resources)
    # 6 of them have sub resources.
    self.assertEquals(6, have_sub_resources)
    # And, of course, 2 should have both sub resources and methods
    self.assertEquals(2, have_sub_resources_and_methods)

  def testParameters(self):
    api = self.ApiFromDiscoveryDoc(self._TEST_DISCOVERY_DOC)
    delete = api.MethodByName('chili.activities.delete')
    self.assertEquals(1, len(delete.query_parameters))
    self.assertEquals(3, len(delete.path_parameters))
    required_p = FindByWireName(delete.values['parameters'],
                                'required_parameter')
    self.assertEquals('query', required_p.location)
    post_id = FindByWireName(delete.values['parameters'], 'postId')
    self.assertEquals('path', post_id.location)

  def testEnums(self):
    gen = self.ApiFromDiscoveryDoc('enums.json')
    # Find the method with the enums
    m1 = gen.MethodByName('language.translations.list')
    language = FindByWireName(m1.values['parameters'], 'language')
    e = language.values['enumType']
    self.assertEquals(m1, e.parent)
    for name, value, desc in e.values['pairs']:
      self.assertTrue(name in ['ENGLISH', 'ITALIAN', 'LANG_ZH_CN',
                               'LANG_ZH_TW'])
      self.assertTrue(value in ['english', 'italian', 'lang_zh-CN',
                                'lang_zh-TW'])
      self.assertTrue(desc in ['English (US)', 'Italian',
                               'Chinese (Simplified)', 'Chinese (Traditional)'])
    accuracy = FindByWireName(m1.values['parameters'], 'accuracy')
    e = accuracy.values['enumType']
    self.assertEquals(m1, e.parent)
    for name, value, desc in e.values['pairs']:
      self.assertTrue(name in ['VALUE_1', 'VALUE_2', 'VALUE_3'])
      self.assertTrue(value in ['1', '2', '3'])

  def testArrayParameter(self):
    api = self.ApiFromDiscoveryDoc(self._TEST_DISCOVERY_DOC)
    search = api.MethodByName('chili.people.search')
    filter_param = FindByWireName(search.values['parameters'], 'filters')
    self.assertTrue(isinstance(filter_param.data_type,
                               data_types.ArrayDataType))
    self.assertTrue(isinstance(filter_param.data_type._base_type,
                               data_types.PrimitiveDataType))
    self.assertEquals('string',
                      filter_param.data_type._base_type.values['type'])

  def testRepeatedEnum(self):
    api = self.ApiFromDiscoveryDoc(self._TEST_DISCOVERY_DOC)
    activities = FindByWireName(api.values['resources'], 'activities')
    list_method = FindByWireName(activities.values['methods'], 'list')
    options = [p for p in list_method.values['parameters']
               if p.values['wireName'] == 'options'][0]
    # Should be an array of enums of type string
    self.assertTrue(isinstance(options.data_type, data_types.ArrayDataType))
    self.assertTrue(isinstance(options.data_type._base_type, data_types.Enum))
    self.assertEquals('string', options.data_type._base_type.values['type'])

  def testScopes(self):
    gen = self.ApiFromDiscoveryDoc(self._TEST_DISCOVERY_DOC)
    scopes = gen.GetTemplateValue('authscopes')
    self.assertEquals(2, len(scopes))
    self.assertEquals('https://www.googleapis.com/auth/buzz',
                      scopes[0].GetTemplateValue('value'))
    self.assertEquals('BUZZ',
                      scopes[0].GetTemplateValue('name'))
    self.assertEquals('https://www.googleapis.com/auth/buzz.read-only',
                      scopes[1].GetTemplateValue('value'))
    self.assertEquals('BUZZ_READ_ONLY',
                      scopes[1].GetTemplateValue('name'))

  def testAuthScope(self):
    api = self.ApiFromDiscoveryDoc(self._TEST_DISCOVERY_DOC)
    scope = AuthScope(api,
                      'https://www.googleapis.com/auth/userinfo.email',
                      {'description': 'A typical scope'})
    self.assertEquals('USERINFO_EMAIL', scope.GetTemplateValue('name'))
    self.assertEquals('userinfo.email', scope.GetTemplateValue('lastPart'))
    self.assertEquals('A typical scope', scope.GetTemplateValue('description'))
    scope = AuthScope(api,
                      'https://www.googleapis.com/auth/no.description', {})
    self.assertEquals('NO_DESCRIPTION', scope.GetTemplateValue('name'))
    self.assertEquals('https://www.googleapis.com/auth/no.description',
                      scope.GetTemplateValue('description'))
    scope = AuthScope(api, 'https://www.googleapis.com/auth/trim.slashes//', {})
    self.assertEquals('TRIM_SLASHES', scope.GetTemplateValue('name'))
    self.assertEquals('https://www.googleapis.com/auth/trim.slashes//',
                      scope.GetTemplateValue('value'))
    scope = AuthScope(api,
                      'https://www.googleapis.com/auth/product',
                      {'description': 'A product level scope'})
    self.assertEquals('PRODUCT', scope.GetTemplateValue('name'))
    scope = AuthScope(api,
                      'https://mail.google.com/',
                      {'description': 'A non-googleapis.com scope'})
    self.assertEquals('MAIL_GOOGLE_COM', scope.GetTemplateValue('name'))
    self.assertEquals('mail.google.com', scope.GetTemplateValue('lastPart'))
    self.assertEquals('https://mail.google.com/',
                      scope.GetTemplateValue('value'))
    scope = AuthScope(api,
                      'https://mail.google.com/abc',
                      {'description': 'A non-googleapis.com scope'})
    self.assertEquals('MAIL_GOOGLE_COM_ABC', scope.GetTemplateValue('name'))
    scope = AuthScope(api,
                      'http://mail.google.com/',
                      {'description': 'A non-https scope'})
    self.assertEquals('HTTP___MAIL_GOOGLE_COM', scope.GetTemplateValue('name'))
    scope = AuthScope(api, 'tag:google.com,2010:auth/groups2#email', {})
    self.assertEquals('TAG_GOOGLE_COM_2010_AUTH_GROUPS2_EMAIL',
                      scope.GetTemplateValue('name'))
    scope = AuthScope(api, 'email', {})
    self.assertEquals('EMAIL', scope.GetTemplateValue('name'))

  def testPostVariations(self):
    gen = self.ApiFromDiscoveryDoc('post_variations.json')
    # Check a normal GET method to make sure it has no request and does have
    # a response
    r1 = FindByWireName(gen.values['resources'], 'r1')
    methods = r1.values['methods']
    m = FindByWireName(methods, 'get')
    self.assertIsNone(m.values['requestType'])
    self.assertEquals('Task', m.values['responseType'].class_name)
    # A normal POST with both a request and response
    m = FindByWireName(methods, 'insert')
    self.assertEquals('Task', m.values['requestType'].class_name)
    self.assertEquals('Task', m.values['responseType'].class_name)
    # A POST with neither request nor response
    m = FindByWireName(methods, 'no_request_no_response')
    self.assertIsNone(m.values.get('requestType'))
    self.assertTrue(isinstance(m.values.get('responseType'), data_types.Void))
    # A POST with no request
    m = FindByWireName(methods, 'no_request')
    self.assertIsNone(m.values.get('requestType'))
    self.assertEquals('Task', m.values['responseType'].class_name)
    # A PUT with no response
    m = FindByWireName(methods, 'no_response')
    self.assertEquals('TaskList', m.values['requestType'].class_name)
    self.assertTrue(isinstance(m.values.get('responseType'), data_types.Void))

  def testSchemaParenting(self):
    api = self.ApiFromDiscoveryDoc(self._TEST_DISCOVERY_DOC)
    # Check that top level schemas have no parent
    for schema in ['Activity', 'Comment']:
      self.assertIsNone(api._schemas[schema].parent)
    for schema in ['Person.urls', 'Activity.object',
                   'Activity.object.attachments']:
      self.assertTrue(api._schemas[schema].parent)
    # verify the values in the name to schema map
    for name, schema in api._schemas.items():
      if schema.parent and schema.parent != api:
        wire_name = schema.values['wireName']
        parent_wire_name = schema.parent.values['wireName']
        # Our entry key should never match the wirename of our parent
        self.assertNotEquals(name, parent_wire_name)
        # our key must look like 'p1.p2....parent.me'. We verify that we at
        # least end with 'parent.me'
        self.assertTrue(name.endswith('.'.join([parent_wire_name, wire_name])))

  def testReadingRpcDiscovery(self):
    gen = self.ApiFromDiscoveryDoc(self._TEST_DISCOVERY_RPC_DOC)
    # no resources in RPC
    self.assertEquals(0, len(gen.values['resources']))
    # but we do expect a few methods
    self.assertLess(5, len(gen.values['methods']))
    self.assertGreater(100, len(gen.values['methods']))
    # RPC methods all have an id, httpMethod should be POST and have no path
    for method in gen.values['methods']:
      self.assertIsNotNone(method.values['id'])
      self.assertEquals('POST', method.values['httpMethod'])
      self.assertIsNone(method.values['restPath'])


  def testNormalizeUrlComponents(self):

    googleapis_base = 'https://www.googleapis.com/'

    def LoadApi(discovery_dict):
      d = {'name': 'fake', 'version': 'v1'}
      d.update(discovery_dict)
      api = Api(d)
      return api

    api = LoadApi({})
    self.assertEquals(googleapis_base, api.values['rootUrl'])
    self.assertEquals('fake/v1/', api.values['servicePath'])

    custom_path = '/testing/fake/v1/'
    api = LoadApi({'basePath': custom_path})
    self.assertEquals(googleapis_base, api.values['rootUrl'])
    self.assertEquals('testing/fake/v1/', api.values['servicePath'])

    custom_url = 'https://foo.com/bar/baz/'
    api = LoadApi({'basePath': custom_url})
    self.assertEquals('https://foo.com/', api.values['rootUrl'])
    self.assertEquals('bar/baz/', api.values['servicePath'])

    # Make sure baseUrl wins over basePath
    api = LoadApi({
        'basePath': '/will/not/be/used/',
        'baseUrl': custom_url
    })
    self.assertEquals('https://foo.com/', api.values['rootUrl'])
    self.assertEquals('bar/baz/', api.values['servicePath'])

    # Make sure rootUrl wins over all
    api = LoadApi({
        'basePath': '/will/not/be/used/',
        'baseUrl': 'https://bar.com/not/used/',
        'rootUrl': 'https://foo.com/',
        'servicePath': 'bar/baz/',
    })
    self.assertEquals('https://foo.com/', api.values['rootUrl'])
    self.assertEquals('bar/baz/', api.values['servicePath'])

    # Test Swarm APIs
    api = LoadApi({
        'baseUrl': 'https://localhost.appspot.com/_ah/api/fake/v1/',
        'basePath': '/_ah/api/fake/v1/',
        'rootUrl': 'https://localhost.appspot.com/_ah/api/',
        'servicePath': 'fake/v1/',
    })
    self.assertEquals('https://localhost.appspot.com/_ah/api/',
                      api.values['rootUrl'])
    self.assertEquals('fake/v1/', api.values['servicePath'])

    # .. in path
    self.assertRaises(ValueError, LoadApi, {'basePath': '/do/not/../go/up'})

    # no servicePath
    self.assertRaises(ValueError, LoadApi, {'rootUrl': 'https://foo.com/'})

  def testCanonicalName(self):
    d = {'name': 'fake', 'version': 'v1', 'canonicalName': 'My API'}
    api = Api(d)
    self.assertEquals('fake', api.values['name'])
    self.assertEquals('MyAPI', api._class_name)

  def testNormalizeOwnerInformation(self):

    def LoadApi(**kwargs):
      d = {'name': 'fake', 'version': 'v1'}
      d.update(kwargs)
      return Api(d)

    api = LoadApi()
    self.assertEquals('Google', api.values['ownerName'])
    self.assertEquals('google', api.values['owner'])
    self.assertEquals('google.com', api.values['ownerDomain'])

    api = LoadApi(ownerName='Google', ownerDomain='youtube.com')
    self.assertEquals('Google', api.values['ownerName'])
    self.assertEquals('google', api.values['owner'])
    self.assertEquals('youtube.com', api.values['ownerDomain'])

    api = LoadApi(ownerDomain='youtube.com')
    self.assertEquals('youtube_com', api.values['owner'])
    self.assertEquals('youtube.com', api.values['ownerDomain'])

    # owner is explicitly declared
    api = LoadApi(owner='You Tube', ownerDomain='youtube.com')
    self.assertEquals('You Tube', api.values['owner'])
    self.assertEquals('youtube.com', api.values['ownerDomain'])

    api = LoadApi(servicePath='/fake',
                  rootUrl='https://www.foobar.co.uk:8080/root')
    self.assertEquals('www.foobar.co.uk', api['ownerDomain'])
    self.assertEquals('www_foobar_co_uk', api['owner'])

    api = LoadApi(servicePath='/fake',
                  rootUrl='https://whathaveyou.googleplex.com')
    self.assertEquals('google.com', api['ownerDomain'])
    self.assertEquals('Google', api['ownerName'])
    self.assertEquals('google', api['owner'])

    api = LoadApi(servicePath='/fake',
                  rootUrl='https://whathaveyou.googleapis.com')
    self.assertEquals('google.com', api['ownerDomain'])
    self.assertEquals('Google', api['ownerName'])
    self.assertEquals('google', api['owner'])

    api = LoadApi(servicePath='/fake',
                  rootUrl='https://whathaveyou.google.com')
    self.assertEquals('google.com', api['ownerDomain'])
    self.assertEquals('Google', api['ownerName'])
    self.assertEquals('google', api['owner'])

  def testSharedTypes(self):
    api = self.ApiFromDiscoveryDoc(self._TEST_SHARED_TYPES_DOC)
    api.VisitAll(lambda o: o.SetLanguageModel(language_model.LanguageModel()))
    # class defined by the API
    photos_feed_schema = api._schemas['PhotosFeed']
    # type defined from a shared type repo
    photo_schema = api._schemas[
        'http://www.googleapis.com/types/v1/com.google/plus/v2/photo']
    self.assertEquals('PhotosFeed', photos_feed_schema.values['wireName'])
    self.assertEquals('com.google.myservice', photos_feed_schema.module.name)
    self.assertEquals('Photo', photo_schema.values['wireName'])
    self.assertEquals('com.google.plus.pictures', photo_schema.module.name)
    self.assertEquals('com/google/plus/pictures', photo_schema.module.path)

  def testMethods(self):
    api = self.ApiFromDiscoveryDoc(self._TEST_DISCOVERY_DOC)
    self.assertEquals(api, api.top_level_methods[0].parent)
    self.assertLess(25, len(api.all_methods))
    self.assertLess(0, len(api.top_level_methods))

  def testApiHasTitle(self):
    api_def = {'name': 'fake',
               'version': 'v1',
               'schemas': {},
               'resources': {}}
    api = Api(api_def)
    self.assertEquals('fake', api['title'])

  def testExponentialBackoffDefault(self):
    # Make sure exponentialBackoffDefault defaults to False.
    discovery_doc = json.loads(
        """
        {
         "name": "fake",
         "version": "v1",
         "schemas": {},
         "resources": {}
        }
        """)
    api = Api(discovery_doc)
    # Make sure exponentialBackoffDefault gets set to True.
    self.assertFalse(api.values['exponentialBackoffDefault'])
    discovery_doc2 = json.loads(
        """
        {
         "name": "fake",
         "version": "v1",
         "schemas": {},
         "resources": {},
         "exponentialBackoffDefault": true
        }
        """)
    api2 = Api(discovery_doc2)
    self.assertTrue(api2.values['exponentialBackoffDefault'])


class ApiModulesTest(basetest.TestCase):

  def setUp(self):
    self.discovery_doc = json.loads(
        """
        {
         "name": "fake",
         "version": "v1",
         "schemas": {},
         "resources": {}
        }
        """)
    self.language_model = FakeLanguageModel()

  def testModuleOwnerDomain(self):
    self.discovery_doc['ownerDomain'] = 'foo.bar'
    api = Api(self.discovery_doc)
    api.VisitAll(lambda o: o.SetLanguageModel(self.language_model))
    self.assertEquals('bar/foo/fake', api.values['module'].path)

  def testModulePackagePath(self):
    self.discovery_doc['packagePath'] = 'foo/BAR'
    api = Api(self.discovery_doc)
    api.VisitAll(lambda o: o.SetLanguageModel(self.language_model))
    self.assertEquals('com/google/foo/BAR/fake', api.values['module'].path)

  def testModuleOwnerDomainAndPackagePath(self):
    self.discovery_doc['ownerDomain'] = 'toasty.com'
    self.discovery_doc['packagePath'] = 'foo/BAR'
    api = Api(self.discovery_doc)
    api.VisitAll(lambda o: o.SetLanguageModel(self.language_model))
    self.assertEquals('com/toasty/foo/BAR/fake', api.values['module'].path)


def FindByWireName(list_of_resource_or_method, wire_name):
  """Find an element in a list by its "wireName".

  The "wireName" is the name of the method "on the wire", which is the raw name
  as it appears in the JSON.

  Args:
    list_of_resource_or_method: A list of resource or methods as annotated by
      the Api.
    wire_name: (str): the name to fine.

  Returns:
    dict or None
  """
  for x in list_of_resource_or_method:
    if x.values['wireName'] == wire_name:
      return x
  return None


if __name__ == '__main__':
  basetest.main()



#!/usr/bin/python2.7
# Copyright 2011 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""A class holding utilities and concepts specific to a programming language.

The LanguageModel class contains conceptual elements which are common to
programming languages, but differ in the details. For example, this class
had the concept of the delimiter between parts of a module. In Java this
would be a '.', in C++ a '::'.
"""
__author__ = 'aiuto@google.com (Tony Aiuto)'

from googleapis.codegen import utilities

# Types of variable name case transforms. E.g. "hello worLd"
PRESERVE_CASE = 0  # hello worLd
LOWER_CASE = 1  # hello world
UPPER_CASE = 2  # HELLO WORLD
LOWER_CAMEL_CASE = 3  # hello WorLd
UPPER_CAMEL_CASE = 4  # Hello WorLd
LOWER_UNCAMEL_CASE = 5  # hello wor ld
UPPER_UNCAMEL_CASE = 6  # HELLO WOR LD
CAP_FIRST = 7  # Hello worLd

# AtSignPolicy - policies for replacing '@' in names
ATSIGN_STRIP = 0  # Strip @
ATSIGN_BREAK = 1  # Treat @ as a word break


class NamingPolicy(object):
  """The policy for transforming a wireName into a language suitable format.

  A naming policy consists of 3 parameters which define the transformation
  of a wireName into a particular type of programming language construct. For
  example, in C++, we might want max-results to be set_max_results when used as
  a setter and _max_results when used as the name of a member variable. The
  parameters are

  case_transform: (enum) The case transform for use for building a name. For
      the input 'hello worlD', the choices yield:
          language_model.PRESERVE_CASE => hello worlD
          language_model.LOWER_CASE => hello world
          language_model.UPPER_CASE => HELLO WORLD
          language_model.LOWER_CAMEL_CASE => hello WorlD
          language_model.UPPER_CAMEL_CASE => Hello WorlD
          language_model.LOWER_UNCAMEL_CASE => hello wor ld
          language_model.UPPER_UNCAMEL_CASE => HELLO WOR LD
  format_string: (str) The format string which wraps the transformed, separated
      name we are building. May be None for the identity format.
  separator: (char) The character which will be used to separate the
      parts of a name. May be None to indicate no separator.
  conflict_policy: (NamingPolicy) If the result of the name is a reserved word
      then try this policy as a fallback (and so on). It is the responsiblity
      of the LanguageModel sub-classer to provide a conflict_policy which
      will always result in a safe name. Note that the conflict_policy may, in
      turn, provide it's own conflict_policy. That allows one to construct a
      chain of formats to try.
  atsign_policy: (enum AtSignPolicy) The policy for handling '@' in names.
  """

  def __init__(self, case_transform=None, format_string=None, separator=None,
               conflict_policy=None, atsign_policy=ATSIGN_STRIP):
    self.case_transform = case_transform
    self.format_string = format_string
    self.separator = separator
    self.conflict_policy = conflict_policy
    self.atsign_policy = atsign_policy


class LanguageModel(object):
  """The base class for all LanguageModels.

  There is a matrix of options of the form {TypeOfName}_{Option}, where the
  nametype specifies the kind of variable name we want and the option
  specifies the case transformation, separator or format. The meaning of
  these three options are:


  The nametypes we define are
     array_of: For making a declaration of an array of some data type.
     class_name: For making a name for a class.
     constant: For making a name for a constant.
     member: For making a class member name.
  """
  # What language this model is for.
  language = ''

  # Characters in addition to alphanumerics which can be in an identifier.
  allowed_characters = ''

  # The defaults for each of (TypeOfName x Option)
  array_of_policy = None  # Note usage in ArrayOf
  map_of_policy = None  # Note usage in MapOf
  class_name_policy = NamingPolicy()
  constant_policy = NamingPolicy()

  # The member, getter, etc... options are a related set. Typically the
  # transforms will all be identical, but the getter/setter will have a format
  # like 'get_{name}'
  member_policy = NamingPolicy()  # the name for inside a class definition
  parameter_name_policy = NamingPolicy()  # the public facing name
  getter_policy = NamingPolicy()
  setter_policy = NamingPolicy()
  unset_policy = NamingPolicy()  # clear the element in the object
  has_policy = NamingPolicy()  # is the element set in the object?
  enum_policy = NamingPolicy()  # What would the element be called if an enum?

  # The list of reserved words in this language. ToXxxName methods must never
  # return one of these.
  reserved_words = []

  def __init__(self, class_name_delimiter='.', module_name_delimiter=None):
    """Create a LanguageModel.

    Args:
      class_name_delimiter: (str) The string which delimits parts of a class
          name in code.
      module_name_delimiter: (str) The string which delimits parts of a module
          name in code. Defaults to class_name_delimiter
    Raises:
      ValueError: if some element of the class is mis-configured.
    """
    super(LanguageModel, self).__init__()
    self._class_name_delimiter = class_name_delimiter
    self._module_name_delimiter = (
        module_name_delimiter or class_name_delimiter)

    # pylint: disable=g-bad-name
    self._SUPPORTED_TYPES = {
        'integer': self._Integer,
        'number': self._Float,
        'string': self._String,
        }
    self._policies_by_name = {
        'class_name': self.class_name_policy,
        'constant': self.constant_policy,
        'getter': self.getter_policy,
        'has': self.has_policy,
        'member': self.member_policy,
        'parameter_name': self.parameter_name_policy,
        'setter': self.setter_policy,
        'unset': self.unset_policy,
        'enum': self.enum_policy,
        }

  def _Integer(self, data_value):
    """Convert provided int to language specific literal.

    Subclasses should override as appropriate.

    Args:
      data_value: The DataValue object representing the value to be converted
        to a integer literal.

    Returns:
      (str) String representation of value once cast to an integer.
    """
    return u'%d' % long(data_value.value)

  def _Float(self, data_value):
    """Convert provided float to language specific literal.

    Args:
      data_value: The DataValue object representing the value to be converted
        to a float literal.

    Returns:
      (str) String representation of value once cast to an float.

    Raises:
      ValueError: if float is inf as we don't support rendering an inf.
    """
    value = float(data_value.value)
    if value == float('inf'):
      raise ValueError('DataValue does not support rendering of provided Type '
                       '(%s)' % value)

    value = unicode(value)
    # Note that unicode(float()) already appends '.0' if is an integral value.
    return value

  def _String(self, data_value):
    """Convert provided string to language specific literal.

    Subclasses should override as appropriate.

    Args:
      data_value: The DataValue object representing the value to be converted
        to a string literal.

    Returns:
      (str) Value written out as a string wrapped in double quotes.
    """
    # TODO(user): Figure out a generalized solution to literal-related
    # information... replicating template_helpers.py escape information here
    # is bad practice.
    literal_escape = [
        ('\\', '\\\\'),
        ('"', '\\"'),
        ('\n', '\\n'),
        ('\t', '\\t'),
        ('\r', '\\r'),
        ('\f', '\\f'),
        ]
    value = unicode(data_value.value)
    for special, replacement in literal_escape:
      value = value.replace(special, replacement)
    return u'"%s"' % value

  @property
  def class_name_delimiter(self):
    return self._class_name_delimiter

  @property
  def module_name_delimiter(self):
    return self._module_name_delimiter

  def GetCodeTypeFromDictionary(self, json_schema):
    """Convert a json schema primitive type into the most suitable class name.

    Subclasses should override as appropriate.

    Args:
      json_schema: (dict) The defintion dictionary for this type
    Returns:
      A name suitable for use as a class in the generator's target language.
    """
    # COV_NF_START
    raise NotImplementedError(
        'Subclasses of LanguageModel must implement GetCodeTypeFromDictionary')
    # COV_NF_END

  def GetPrimitiveTypeFromDictionary(self, unused_json_schema):
    """Convert a json schema primitive type into a language primitive.

    Subclasses should override as appropriate. This should be the primitive
    form of the result from GetCodeTypeFromDictionary(), or None if no primitive
    exists.

    Args:
      unused_json_schema: (dict) The defintion dictionary for this type
    Returns:
      A name suitable for use as a class in the generator's target language.
    """
    return None

  def ArrayOf(self, variable, type_name):
    """Produce the string declaring an array of a data type.

    Args:
      variable: (DataType) the data we want an array of.
      type_name: (str) the printable name of that data type. Usually
          variable.codeType.
    Returns:
      (str)
    """
    if self.array_of_policy:
      return self.ApplyFormat(variable, type_name, self.array_of_policy)
    # COV_NF_START
    raise NotImplementedError(
        'Subclasses of LanguageModel must implement ArrayOf or provide'
        ' array_of_policy')
    # COV_NF_END

  def MapOf(self, variable, type_name):
    """Produce the string declaring a map of string to a data type.

    Args:
      variable: (DataType) the data we want an array of.
      type_name: (str) the printable name of that data type. Usually
          variable.codeType.
    Returns:
      (str)
    """
    if self.map_of_policy:
      return self.ApplyFormat(variable, type_name, self.map_of_policy)
    # COV_NF_START
    raise NotImplementedError(
        'Subclasses of LanguageModel must implement MapOf or provide'
        ' map_of_policy')
    # COV_NF_END

  def CodeTypeForVoid(self):
    """Return the type name for a void.

    Subclasses may override this default implementation.

    Returns:
      (str) 'void'
    """
    return 'void'

  def ApplyPolicy(self, policy_name, variable, name):
    """Apply a naming policy to a string.

    Maps the policy name to the tranformation class and applies it.

    Args:
      policy_name: (str) The name of a policy.
      variable: (CodeObject) an element which may appear in the templates,
          but typically a Property.
      name: (str) The Discovery name of the variable.
    Returns:
      (str) The input string transformed as specified by the policy.
    """
    policy = self._policies_by_name[policy_name]
    return self.TransformString(variable, name, policy)

  def ToMemberName(self, s, api):  # pylint: disable=unused-argument
    """Convert a name to a suitable member name in the target language.

    Subclasses should override as appropriate.

    TODO(user): Make a pass to replace uses of this with ToMemberName or
    whatever is appropriate,

    Args:
      s: (str) A canonical name for data element. (Usually the API wire format)
      api: (Api) The API this element is part of. For use as a hint when the
        name cannot be used directly.
    Returns:
      A name suitable for use as an element in the generator's target language.
    """
    return s

  def ToSafeClassName(self, s, api, parent):  # pylint: disable=unused-argument
    """Convert a name to a suitable class name in the target language.

    Subclasses should override as appropriate.

    Args:
      s: (str) A canonical name for data element. (Usually the API wire format)
      api: (Api) The API this element is part of. For use as a hint when the
        name cannot be used directly.
      parent: The schema where I was referenced.
    Returns:
      A name suitable for use as an element in the generator's target language.
    """
    return utilities.CamelCase(s)

  def ToPropertyGetterMethodWithDelim(self, prop_name):
    """Convert a property name to the name of the getter method that returns it.

    Subclasses should override as appropriate.

    Args:
      prop_name: (str) The name of a property.
    Returns:
      The language specific name of the getter method that returns the specified
      property. The default implementation returns .getxyz for a property called
      xyz.
    """
    return '%sget%s()' % (self._class_name_delimiter, prop_name)

  def RenderDataValue(self, data_value):
    """Translate a value to an appropriate structure for the target language.

    Args:
      data_value: The DataValue object rendered.
    Returns:
      The string representation of the value the target language expects
      of the type represented by the schema (e.g. a value [1, 2, 3] with
      a schema representing a list would be translated to "[1, 2, 3]" in
      Python but "Arrays.asList({1, 2, 3})" in Java).

    Raises:
      ValueError: if the provided schema is for an unsupported type.
    """
    type_str = data_value.data_type.json_type
    if type_str in self._SUPPORTED_TYPES:
      return self._SUPPORTED_TYPES[type_str](data_value)
    else:
      raise ValueError(
          'Rendering the provided type (%s) is not supported by the %s.' %
          (type_str, type(self).__name__))

  # pylint: disable=unused-argument
  def DefaultContainerPathForOwner(self, module):
    """Computes the default path to a module for the given owner information.

    For a given module, compute the portions of the path which are determined
    by the owner of the module - excluding the module path itself.

    Subclasses almost certainly want to override this.

    Args:
      module: (template_objects.Module) The module.
    Returns:
      (str) The path to use for the namespace of this API.
    """
    return '/'.join(utilities.ReversedDomainComponents(module.owner_domain))

  def ApplyCaseTransform(self, s, policy):
    """Applies a Policy's case transforms to a string.

    Args:
      s: (str) A string to transform.
      policy: (NamingPolicy) The naming policy to use for the transform.
    Returns:
      Case transformed string.
    """
    # Pick the right transformer method
    if policy.case_transform == LOWER_CASE:
      transform = lambda s, _: ''.join(s).lower()
    elif policy.case_transform == UPPER_CASE:
      transform = lambda s, _: ''.join(s).upper()
    elif policy.case_transform == UPPER_CAMEL_CASE:
      transform = lambda s, _: ''.join([s[0].upper()] + s[1:])
    elif policy.case_transform == LOWER_CAMEL_CASE:
      # pylint: disable=g-long-lambda
      transform = (
          lambda s, first_word:
          ''.join([s[0].lower() if first_word else s[0].upper()] + s[1:]))
    elif policy.case_transform == LOWER_UNCAMEL_CASE:
      transform = lambda s, _: utilities.UnCamelCase(''.join(s))
    elif policy.case_transform == UPPER_UNCAMEL_CASE:
      transform = lambda s, _: utilities.UnCamelCase(''.join(s)).upper()
    elif policy.case_transform == CAP_FIRST:
      # pylint: disable=g-long-lambda
      transform = (
          lambda s, first_word:
          ''.join([s[0].upper() if first_word else s[0]] + s[1:]))
    else:
      transform = lambda s, _: ''.join(s)

    if policy.atsign_policy == ATSIGN_STRIP:
      s = s.replace('@', '')
    if policy.atsign_policy == ATSIGN_BREAK:
      # chr(1) is always going to not be in allowed_characters.
      s = s.replace('@', policy.separator or chr(1))

    # Split into words at characters which can not be part of an identifier.
    parts = []
    curpart = []
    for c in s:
      # Collect valid characters for an identifier.
      if not c.isalnum() and c not in self.allowed_characters:
        if curpart:
          parts.append(transform(curpart, not parts))
          curpart = []
      else:
        # end the word when we have a bad one.
        curpart.append(c)
    if curpart:
      parts.append(transform(curpart, not parts))

    join_char = policy.separator or ''
    return join_char.join(parts)

  def TransformString(self, variable, s, policy):
    """Applies the transforms of a naming policy to a string.

    Takes a string (usually a wireName) which might be in any case and have
    reserved characters in it and transforms by the rules specified. The string
    is divided into parts at reserved characters, then each part is transformed
    by the case rule and then the parts are joined by the reserved character
    replacement.  Multiple reserved characters in a row are treated as one.

    Note that the camel case transformations preserve existing case except for
    the first character of each word. This provides good results for cases
    where the use has provided a camel cased or proper name. E.g. maxResults,
    YouTube, NASA.

    Note: Do we need rule that transforms NASA to Nasa and nasa when in the
    upper and lower camel variations?  That is, if you specify something in
    ALL CAPS, we assume it is not a mixed case spelling.

    Args:
      variable: (CodeObject) The template variable this string came from. This
          is used to extract details about the variable which may be useful in
          building a name, such as the module it belongs to.
      s: (str) A string to transform.
      policy: (NamingPolicy) The naming policy to use for the transform.
    Returns:
      Transformed string.
    """
    name = self.ApplyCaseTransform(s, policy)
    if policy.format_string:
      name = self.ApplyFormat(variable, name, policy)

    if name.lower() in self.reserved_words:
      if policy.conflict_policy:
        return self.TransformString(variable, s, policy.conflict_policy)
      else:
        return name + '__'  # Ugly, but works everywhere.
    return name

  def ApplyFormat(self, variable, name, policy):
    """Applies the format of a naming policy to a string.

    Args:
      variable: (CodeObject) The template variable this string came from. This
          is used to extract details about the variable which may be useful in
          building a name, such as the module it belongs to.
      name: (str) A name to transform.
      policy: (NamingPolicy) The naming policy to use for the transform.
    Returns:
      Transformed string.
    """
    expansions = dict(name=name)
    # The variable should always be present in normal execution. We allow
    # it to be None solely for testing.
    # TODO(user): Figure out a way to fail hard if not present during
    # execution, even though django catches the error and siliently ignores it.
    # OR... maybe just fix the tests to always pass in the complete kind of
    # object we need.
    parent_name = 'global'
    if variable:
      if hasattr(variable, 'module'):
        expansions['module'] = variable.module.name
      if hasattr(variable, 'api'):
        api = variable.api
        if api:
          api_name = api.GetTemplateValue('wireName') or parent_name
        expansions['api_name'] = self.ApplyCaseTransform(api_name, policy)
      if hasattr(variable, 'parent'):
        parent = variable.parent
        if parent:
          parent_name = parent.GetTemplateValue('wireName') or parent_name
      if hasattr(variable, 'schema'):
        # This lets the format string access all template values on schema,
        # which is powerful but potentially flaky if the schema changes.
        expansions['schema'] = variable.schema
      expansions['variable_name'] = self.ApplyCaseTransform(
          variable.GetTemplateValue('wireName') or '', policy)
    expansions['parent_name'] = self.ApplyCaseTransform(parent_name,
                                                        policy)
    # TODO(user): Expand the range of things available.
    return policy.format_string.format(**expansions)


class DocumentingLanguageModel(LanguageModel):
  """A language model which displays things in a way humans can read.

  This model is useful for language neutral expression of an Api.
  """
  array_of_policy = NamingPolicy(format_string='Array<{name}>')
  map_of_policy = NamingPolicy(format_string='Map<string, {name}>')
  class_name_policy = NamingPolicy(case_transform=UPPER_CAMEL_CASE)

  def GetCodeTypeFromDictionary(self, json_schema):
    """Convert a json schema primitive type into the most suitable class name.

    Subclasses should override as appropriate.

    Args:
      json_schema: (dict) The defintion dictionary for this type
    Returns:
      A name suitable for use as a class in the generator's target language.
    """
    json_type = json_schema.get('type', 'UNKNOWN')
    json_format = json_schema.get('format')
    if json_format:
      json_type = '%s (%s)' % (json_type, json_format)
    return json_type



#!/usr/bin/python2.7
# Copyright 2012 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Objects for holding data values which can be expanded in templates.

This module contains the base classes for objects which can hold data values
(scaler constants, arrays, ...) that might be expanded in a template. It is
primarily used for CSP.
"""

from googleapis.codegen import template_objects


class DataValue(template_objects.CodeObject):
  """Provide a reasonable value wrapper for converting types to strings."""

  def __init__(self, value, val_type):
    # Because val_type could be a schema and DataObject tries to deepcopy
    # the definition dictionary when schemas store CodeObjects in their
    # _def_dicts. Sidestep this part.
    super(DataValue, self).__init__(
        {}, api=val_type.api, parent=val_type.parent)
    self._def_dict = val_type.values

    # Type may be passed in wrapped in a Property/Parameter object...
    if hasattr(val_type, 'data_type'):
      data_type = val_type.data_type
    else:
      data_type = val_type

    # Type may be a reference, we want the real thing...
    # TODO(user): 20130227
    # If you need this trickery here, we are doing something wrong in the
    # overall structure.
    data_type = getattr(data_type, 'referenced_schema', data_type)

    self._value = value
    self._data_type = data_type
    self._metadata = {}

  def SetValue(self, value):
    self._value = value

  def SetLanguageModel(self, language_model):
    super(DataValue, self).SetLanguageModel(language_model)
    self.data_type.SetLanguageModel(language_model)

  def GetLanguageModel(self):
    # pylint: disable=protected-access
    return self.data_type.language_model

  @property
  def value(self):
    return self._value

  @property
  def data_type(self):
    return self._data_type

  @property
  def code_type(self):
    # Certain data types return a code_type dependent on language model...
    # so we assure that our data_type uses the correct one for safety.
    if self._language_model:
      self._data_type.SetLanguageModel(self._language_model)
    return self._data_type.code_type

  @property
  def metadata(self):
    return self._metadata



#!/usr/bin/python2.7
# Copyright 2010 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for template_objects.py."""

__author__ = 'akesling@google.com (Alex Kesling)'

import json

from google.apputils import basetest

from googleapis.codegen import data_types
from googleapis.codegen import data_value
from googleapis.codegen import python_generator


class PythonLanguageModelDataValueTest(basetest.TestCase):

  def setUp(self):
    self.language_model = python_generator.PythonLanguageModel()

  def _CreateDataValue(self, value, val_type):
    def_dict = {
        'className': 'Foo',
        'type': val_type,
        }
    prototype = data_types.DataType(
        def_dict, None, language_model=self.language_model)
    dv = data_value.DataValue(value, prototype)
    return dv

  def testRenderBoolean(self):
    dv = self._CreateDataValue(True, 'boolean')
    render_method = self.language_model._SUPPORTED_TYPES['boolean']
    self.assertEqual('True', render_method(dv))

    dv.SetValue(False)
    self.assertEqual('False', render_method(dv))

  def testRenderDictionary(self):
    bar_dv = self._CreateDataValue('bar"', 'string')
    qux_dv = self._CreateDataValue('\\"qux', 'string')
    dv = self._CreateDataValue({'foo': bar_dv, 'baz': qux_dv}, 'object')
    render_method = self.language_model._SUPPORTED_TYPES['object']
    expected = {'foo': 'bar"', 'baz': '\\"qux'}
    got = json.loads(render_method(dv))
    self.assertEqual(expected, got)

  def testRenderList(self):
    bar_dv = self._CreateDataValue('bar"', 'string')
    qux_dv = self._CreateDataValue('\\"qux', 'string')
    dv = self._CreateDataValue([bar_dv, qux_dv], 'array')
    render_method = self.language_model._SUPPORTED_TYPES['array']
    self.assertEqual(
        '["bar\\"", "\\\\\\"qux"]', render_method(dv))

if __name__ == '__main__':
  basetest.main()



#!/usr/bin/python2.7
# Copyright 2011 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Wrapper methods to insulate us from Django nuances.

Provide Django setup and some utility methods.
"""

__author__ = 'aiuto@google.com (Tony Aiuto)'

import os



from django import template as django_template
from django import utils as django_utils
from django.conf import settings
# COV_NF_START
try:
  # In AppEngine, we have to call use_library() in our main. Doing that causes
  # an error, which we can safely ignore because use_library did it
  settings.configure()
  os.environ['DJANGO_SETTINGS_MODULE'] = 'settings'
except RuntimeError:
  pass

from googleapis.codegen import template_helpers
from googleapis.codegen.filesys import files


# COV_NF_END

# This is Django magic to add builtin tags and filters.  They don't really
# support that use case.  Instead you are supposed to put a package of filters
# in a specific place and the Django web server finds them for you. We are a
# standalone app, not running in their context, so we have to go under the hood
# a little.
django_template.base.add_to_builtins(
    'googleapis.codegen.template_helpers')


def DjangoRenderTemplate(template_path, context_dict):
  """Renders a template specified by a file path with a give values dict.

  Args:
    template_path: (str) Path to file.
    context_dict: (dict) The dictionary to use for template evaluation.
  Returns:
    (str) The expanded template.
  """

  source = files.GetFileContents(template_path).decode('utf-8')
  return _DjangoRenderTemplateSource(source, context_dict)


def _DjangoRenderTemplateSource(template_source, context_dict):
  """Renders the given template source with the given values dict.

  Args:
    template_source: (str) The source of a django template.
    context_dict: (dict) The dictionary to use for template evaluation.
  Returns:
    (str) The expanded template.
  """
  t = django_template.Template(template_source)
  ctxt = django_template.Context(context_dict)
  with template_helpers.SetCurrentContext(ctxt):
    return t.render(ctxt)


def MarkSafe(s):
  return django_utils.safestring.mark_safe(s)



#!/usr/bin/python2.7
# Copyright 2012 Google Inc. All Rights Reserved.

"""Tests for generator_lookup."""

__author__ = 'akesling@google.com (Alex Kesling)'

import json
import os


from google.apputils import basetest

from googleapis.codegen import generator_lookup
from googleapis.codegen import gwt_generator
from googleapis.codegen import java_generator
from googleapis.codegen import targets


class GeneratorLookupTest(basetest.TestCase):

  def testDetermineGenerator(self):
    test_gen = generator_lookup.GetGeneratorByLanguage('java')
    self.assertEqual(java_generator.Java14Generator, test_gen)
    self.assertRaises(
        ValueError, generator_lookup.GetGeneratorByLanguage,
        'I\'m an invalid language!')

  def testSupportedLanguage(self):
    languages = generator_lookup.SupportedLanguages()
    self.assertContainsSubset(['dart', 'gwt', 'java', 'php'], languages)
    self.assertNotIn('java-head', languages)

  def testVersionFromFeature(self):
    template_root = os.path.join(os.path.dirname(__file__),
                                 'testdata/languages')
    targets.Targets.SetDefaultTemplateRoot(template_root)
    features_path = os.path.join(template_root,
                                 'java/generator_test/features.json')
    raw_features = json.load(open(features_path))
    generator_name = raw_features['generator']
    gen = generator_lookup.GetGeneratorByLanguage(generator_name)
    self.assertEquals(gwt_generator.GwtGenerator, gen)


if __name__ == '__main__':
  basetest.main()



#!/usr/bin/python2.7


from google.apputils import basetest
from googleapis.codegen.utilities import convert_size


class ConvertSizeTest(basetest.TestCase):
  """Test for unit conversion."""

  def testConvertSize(self):
    self.assertEquals(None, convert_size.ConvertSize(None))
    self.assertEquals(None, convert_size.ConvertSize('4'))
    self.assertEquals(None, convert_size.ConvertSize('4C'))
    self.assertEquals(4, convert_size.ConvertSize('4B'))
    self.assertEquals(4 * 2 ** 10, convert_size.ConvertSize('4KB'))
    self.assertEquals(12 * 2 ** 20, convert_size.ConvertSize('12MB'))
    self.assertEquals(10 * 2 ** 30, convert_size.ConvertSize('10GB'))


if __name__ == '__main__':
  basetest.main()



#!/usr/bin/python2.7
# Copyright 2010 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""A LibraryPackage that writes to the file system.

This module implements the LibraryPackage interface, but writes directly to the
file system.
"""

__author__ = 'aiuto@google.com (Tony Aiuto)'

import os

from googleapis.codegen.filesys.library_package import LibraryPackage


class FilesystemLibraryPackage(LibraryPackage):
  """The library package."""

  def __init__(self, root_path):
    """Create a new FilesystemLibraryPackage.

    Args:
      root_path: (str) A path to a directory where the files will be written.
        The directory will be created if it does not exist.
    Raises:
      ValueError: If the directory exists, but is not writable.
      OSError: If the directory does not exist and cannot be created.
    """
    super(FilesystemLibraryPackage, self).__init__()
    # Create the directory if we have to
    self._MakePath(root_path)
    self._root_path = root_path
    self._current_file_stream = None

  def StartFile(self, name):
    """Start writing a named file to the package.

    Args:
      name: (str) path which will identify the contents in the archive.

    Returns:
      A file-like object to write the contents to.
    """
    self.EndFile()
    full_path = os.path.join(self._root_path, self._file_path_prefix, name)
    self._MakePath(os.path.dirname(full_path))
    self._current_file_stream = open(full_path, 'w')
    return self._current_file_stream

  def EndFile(self):
    """Flush the current output file."""
    if self._current_file_stream:
      self._current_file_stream.close()
      self._current_file_stream = None

  def _MakePath(self, path):
    """Create a directory path if needed.

    Args:
      path: (str) A path to a directory where files will be written.  The
        directory will be created if it does not exist.
    Raises:
      ValueError: If the directory exists, but is not writable.
      OSError: If the directory does not exist and cannot be created.
    """
    if not os.access(path, os.W_OK):
      if os.access(path, os.F_OK):
        raise ValueError('%s exists, but is not writable' % path)
      os.makedirs(path, 0755)



#!/usr/bin/python2.7
# Copyright 2010 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""C++ library generator's import manager."""

__author__ = 'ewiseblatt@google.com (Eric Wiseblatt)'

from googleapis.codegen.utilities import ordered_set


class CppImportManager(object):
  """The import manager for the C++ code generator."""

  def __init__(self, element):
    """Construct an import manager for the specified element.

    Args:
      element: (Schema) or (Api). The element we want to create an import
        manager for.
    """
    self._element = element
    self._element.SetTemplateValue('importManager', self)
    self._class_name_to_qualified_name = {}
    self._google_imports = ordered_set.MutableOrderedSet()
    self._platform_imports = ordered_set.MutableOrderedSet()
    self._other_imports = ordered_set.MutableOrderedSet()
    self._type_dependencies = ordered_set.MutableOrderedSet()

  @classmethod
  def ForElement(cls, element):
    manager = element.get('importManager')
    if manager:
      return manager
    # instantiation installs manager inside element.
    return cls(element)

  def ImportLists(self):
    """Returns the set of import lists."""
    # We're returning a set of two rather than three because we're considering
    # the google imports part of the platform when generating the headers.
    # We're avoiding the line breaks the import template forces between llists.
    return [sorted(self._platform_imports) + sorted(self._google_imports),
            sorted(self._other_imports)]

  def AddDataType(self, data_type):
    self._type_dependencies.add(data_type)
    return self.AddImport('"%s"' % data_type.values['include_path'])

  def AddImport(self, fully_qualified_class):
    """Adds the specified import to the import manager.

    Args:
      fully_qualified_class: (str) The fully qualified class we want to add to
        this instance of import manager.

    Returns:
      True- If adding the import is successful.
      False- If adding the import results in a name collision.
    """
    # check to see if it already exists
    class_name = self.GetClassName(fully_qualified_class)
    current_import = self._class_name_to_qualified_name.get(class_name)
    if current_import:
      return current_import == fully_qualified_class
    if fully_qualified_class.startswith('"googleapis/'):
      self._google_imports.add(fully_qualified_class)
    elif self.IsPlatformClass(fully_qualified_class):
      self._platform_imports.add(fully_qualified_class)
    else:
      self._other_imports.add(fully_qualified_class)
    # Now add it to the map
    self._class_name_to_qualified_name[class_name] = fully_qualified_class
    return True

  def IsPlatformClass(self, fully_qualified_class):
    return fully_qualified_class.startswith('<')

  def GetClassName(self, fully_qualified_class):
    return fully_qualified_class.split('::')[-1]

  @property
  def type_dependencies(self):
    return self._type_dependencies

  @property
  def google_imports(self):
    return self._google_imports

  @property
  def platform_imports(self):
    return self._platform_imports

  @property
  def other_imports(self):
    return self._other_imports



#!/usr/bin/python2.7
# Copyright 2010 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for api_exception.py."""

from google.apputils import basetest

from googleapis.codegen.api_exception import ApiException


class ApiExceptionTest(basetest.TestCase):

  def testExceptionStr(self):
    e = ApiException('foo')
    self.assertEquals('foo', str(e))
    e = ApiException('foo', {'bar': 1})
    self.assertEquals("""foo: {'bar': 1}""", str(e))


if __name__ == '__main__':
  basetest.main()



#!/usr/bin/python2.7
# Copyright 2011 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Convert human readable sizes to numbers.

Convert a string like 10G/K/M/B to a number.
"""


def ConvertSize(size):
  if not size:
    return None
  units = [('GB', 2 ** 30),
           ('MB', 2 ** 20),
           ('KB', 2 ** 10),
           ('B', 1)]
  size = size.upper()
  for suffix, multiplier in units:
    if size.endswith(suffix):
      num_units = size[:-len(suffix)]
      try:
        return int(num_units) * multiplier
      except (ValueError, KeyError):
        break
  return None



#!/usr/bin/python2.7
# Copyright 2010 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""API data models - schemas and their properties.

This module handles the objects created for the "schema" section of an API.
"""

__author__ = 'aiuto@google.com (Tony Aiuto)'

import collections
import logging


from googleapis.codegen import data_types
from googleapis.codegen import template_objects
from googleapis.codegen.api_exception import ApiException


_ADDITIONAL_PROPERTIES = 'additionalProperties'

_LOGGER = logging.getLogger('codegen')


class Schema(data_types.ComplexDataType):
  """The definition of a schema."""

  def __init__(self, api, default_name, def_dict, parent=None):
    """Construct a Schema object from a discovery dictionary.

    Schemas represent data models in the API.

    Args:
      api: (Api) the Api instance owning the Schema
      default_name: (str) the default name of the Schema. If there is an 'id'
        member in the definition, that is used for the name instead.
      def_dict: (dict) a discovery dictionary
      parent: (Schema) The containing schema. To be used to establish unique
        names for anonymous sub-schemas.
    """
    super(Schema, self).__init__(default_name, def_dict, api, parent=parent)

    name = def_dict.get('id', default_name)
    _LOGGER.debug('Schema(%s)', name)

    # Protect against malicious discovery
    template_objects.CodeObject.ValidateName(name)
    self.SetTemplateValue('wireName', name)
    class_name = api.ToClassName(name, self, element_type='schema')
    self.SetTemplateValue('className', class_name)
    self.SetTemplateValue('isSchema', True)
    self.SetTemplateValue('properties', [])
    self._module = (template_objects.Module.ModuleFromDictionary(self.values)
                    or api.model_module)

  @classmethod
  def Create(cls, api, default_name, def_dict, wire_name, parent=None):
    """Construct a Schema or DataType from a discovery dictionary.

    Schemas contain either object declarations, simple type declarations, or
    references to other Schemas.  Object declarations conceptually map to real
    classes.  Simple types will map to a target language built-in type.
    References should effectively be replaced by the referenced Schema.

    Args:
      api: (Api) the Api instance owning the Schema
      default_name: (str) the default name of the Schema. If there is an 'id'
        member in the definition, that is used for the name instead.
      def_dict: (dict) a discovery dictionary
      wire_name: The name which will identify objects of this type in data on
        the wire. The path of wire_names can trace an item back through
        discovery.
      parent: (Schema) The containing schema. To be used to establish nesting
        for anonymous sub-schemas.

    Returns:
      A Schema or DataType.

    Raises:
      ApiException: If the definition dict is not correct.
    """

    schema_id = def_dict.get('id')
    if schema_id:
      name = schema_id
    else:
      name = default_name
    class_name = api.ToClassName(name, None, element_type='schema')

    _LOGGER.debug('Create: %s, parent=%s', name,
                  parent.values.get('wireName', '<anon>') if parent else 'None')

    # Schema objects come in several patterns.
    #
    # 1. Simple objects
    #    { type: object, properties: { "foo": {schema} ... }}
    #
    # 2. Maps of objects
    #    { type: object, additionalProperties: { "foo": {inner_schema} ... }}
    #
    #    What we want is a data type which is Map<string, {inner_schema}>
    #    The schema we create here is essentially a built in type which we
    #    don't want to generate a class for.
    #
    # 3. Arrays of objects
    #    { type: array, items: { inner_schema }}
    #
    #    Same kind of issue as the map, but with List<{inner_schema}>
    #
    # 4. Primitive data types, described by type and format.
    #    { type: string, format: int32 }
    #    { type: string, enum: ["value", ...], enumDescriptions: ["desc", ...]}
    #
    # 5. Refs to another schema.
    #    { $ref: name }
    #
    # 6. Variant schemas
    #    { type: object, variant: { discriminant: "prop", map:
    #             [ { 'type_value': value, '$ref': wireName }, ... ] } }
    #
    #    What we do is map the variant schema to a schema with a single
    #    property for the discriminant. To that property, we attach
    #    the variant map which specifies which discriminator values map
    #    to which schema references. We also collect variant information
    #    in the api so we can later associate discriminator value and
    #    base type with the generated variant subtypes.

    if 'type' in def_dict:
      # The 'type' field of the schema can either be 'array', 'object', or a
      # base json type.
      json_type = def_dict['type']
      if json_type == 'object':

        # Look for variants
        variant = def_dict.get('variant')
        if variant:
          return cls._CreateVariantType(variant, api, name,
                                        def_dict, wire_name, parent)

        # Look for full object definition.  You can have properties or
        # additionalProperties, but it does not  do anything useful to have
        # both.

        # Replace properties dict with Property's
        props = def_dict.get('properties')
        if props:
          # This case 1 from above
          return cls._CreateObjectWithProperties(props, api, name,
                                                 def_dict, wire_name, parent)

        # Look for case 2
        additional_props = def_dict.get(_ADDITIONAL_PROPERTIES)
        if additional_props:
          return cls._CreateMapType(additional_props, api, name, wire_name,
                                    class_name, parent)

        # no properties
        return cls._CreateSchemaWithoutProperties(api, name, def_dict,
                                                  wire_name, parent)

      elif json_type == 'array':
        # Case 3: Look for array definition
        return cls._CreateArrayType(api, def_dict, wire_name, class_name,
                                    schema_id, parent)
      else:
        # Case 4: This must be a basic type.  Create a DataType for it.
        return data_types.CreatePrimitiveDataType(def_dict, api, wire_name,
                                                  parent=parent)

    referenced_schema = def_dict.get('$ref')
    if referenced_schema:
      # Case 5: Reference to another Schema.
      #
      # There are 4 ways you can see '$ref' in discovery.
      # 1. In a property of a schema or a method request/response, pointing
      #    back to a previously defined schema
      # 2. As above, pointing to something not defined yet.
      # 3. In a method request or response or property of a schema pointing to
      #    something undefined.
      #
      # For case 1, the schema will be in the API name to schema map.
      #
      # For case 2, just creating this placeholder here is fine.  When the
      # actual schema is hit in the loop in _BuildSchemaDefinitions, we will
      # replace the entry and DataTypeFromJson will resolve the to the new def.
      #
      # For case 3, we will end up with a dangling reference and fail later.
      schema = api.SchemaByName(referenced_schema)
      # The stored "schema" may not be an instance of Schema, but rather a
      # data_types.PrimitiveDataType, which has no 'wireName' value.
      if schema:
        _LOGGER.debug('Schema.Create: %s => %s',
                      default_name, schema.values.get('wireName', '<unknown>'))
        return schema
      return data_types.SchemaReference(referenced_schema, api)

    raise ApiException('Cannot decode JSON Schema for: %s' % def_dict)

  @classmethod
  def _CreateObjectWithProperties(cls, props, api, name, def_dict,
                                  wire_name, parent):
    properties = []
    schema = cls(api, name, def_dict, parent=parent)
    if wire_name:
      schema.SetTemplateValue('wireName', wire_name)
    for prop_name in sorted(props):
      prop_dict = props[prop_name]
      _LOGGER.debug('  adding prop: %s to %s', prop_name, name)
      properties.append(Property(api, schema, prop_name, prop_dict))
      # Some APIs express etag directly in the response, others don't.
      # Knowing that we have it explicitly makes special case code generation
      # easier
      if prop_name == 'etag':
        schema.SetTemplateValue('hasEtagProperty', True)
    schema.SetTemplateValue('properties', properties)

    # check for @ clashing. E.g. No 'foo' and '@foo' in the same object.
    names = set()
    for p in properties:
      wire_name = p.GetTemplateValue('wireName')
      no_at_sign = wire_name.replace('@', '')
      if no_at_sign in names:
        raise ApiException(
            'Property name clash in schema %s:'
            ' %s conflicts with another property' % (name, wire_name))
      names.add(no_at_sign)

    return schema

  @classmethod
  def _CreateVariantType(cls, variant, api, name, def_dict,
                         wire_name, parent):
    """Creates a variant type."""
    variants = collections.OrderedDict()
    schema = cls(api, name, def_dict, parent=parent)
    if wire_name:
      schema.SetTemplateValue('wireName', wire_name)
    discriminant = variant['discriminant']

    # Walk over variants building the variant map and register
    # variant info on the api.
    for variant_entry in variant['map']:
      discriminant_value = variant_entry['type_value']
      variant_schema = api.DataTypeFromJson(variant_entry, name, parent=parent)
      variants[discriminant_value] = variant_schema
      # Set variant info. We get the original wire name from the JSON properties
      # via '$ref' it is not currently accessible via variant_schema.
      api.SetVariantInfo(variant_entry.get('$ref'), discriminant,
                         discriminant_value, schema)

    prop = Property(api, schema, discriminant, {'type': 'string'},
                    key_for_variants=variants)
    schema.SetTemplateValue('is_variant_base', True)
    schema.SetTemplateValue('discriminant', prop)
    schema.SetTemplateValue('properties', [prop])
    return schema

  @classmethod
  def _CreateMapType(cls, additional_props, api, name, wire_name,
                     class_name, parent):
    _LOGGER.debug('Have only additionalProps for %s, dict=%s',
                  name, additional_props)
    # TODO(user): Remove this hack at the next large breaking change
    # The "Items" added to the end is unneeded and ugly. This is for
    # temporary backwards compatibility.  Same for _CreateArrayType().
    if additional_props.get('type') == 'array':
      name = '%sItem' % name
    subtype_name = additional_props.get('id', name + 'Element')
    # Note, since this is an interim, non class just to hold the map
    # make the parent schema the parent passed in, not myself.
    _LOGGER.debug('name:%s, wire_name:%s, subtype name %s', name, wire_name,
                  subtype_name)
    # When there is a parent, we synthesize a wirename when none exists.
    # Purpose is to avoid generating an extremely long class name, since we
    # don't do so for other nested classes.
    if parent and wire_name:
      base_wire_name = wire_name + 'Element'
    else:
      base_wire_name = None

    base_type = api.DataTypeFromJson(
        additional_props, subtype_name, parent=parent,
        wire_name=base_wire_name)
    map_type = data_types.MapDataType(name, base_type, parent=parent,
                                      wire_name=wire_name)
    map_type.SetTemplateValue('className', class_name)
    _LOGGER.debug('  %s is MapOf<string, %s>',
                  class_name, base_type.class_name)
    return map_type

  @classmethod
  def _CreateSchemaWithoutProperties(cls, api, name, def_dict, wire_name,
                                     parent):
    if parent:
      # code objects have __getitem__(), but not .get()
      try:
        pname = parent['id']
      except KeyError:
        pname = '<unknown>'
      name_to_log = '%s.%s' % (pname, name)
    else:
      name_to_log = name
    logging.warning('object without properties %s: %s',
                    name_to_log, def_dict)
    schema = cls(api, name, def_dict, parent=parent)
    if wire_name:
      schema.SetTemplateValue('wireName', wire_name)
    return schema

  @classmethod
  def _CreateArrayType(cls, api, def_dict, wire_name,
                       class_name, schema_id, parent):
    items = def_dict.get('items')
    if not items:
      raise ApiException('array without items in: %s' % def_dict)
    tentative_class_name = class_name
    # TODO(user): THIS IS STUPID. We should not rename things items.
    # if we have an anonymous type within a map or array, it should be
    # called 'Item', and let the namespacing sort it out.
    if schema_id:
      _LOGGER.debug('Top level schema %s is an array', class_name)
      tentative_class_name += 'Items'
    base_type = api.DataTypeFromJson(items, tentative_class_name,
                                     parent=parent, wire_name=wire_name)
    _LOGGER.debug('  %s is ArrayOf<%s>', class_name, base_type.class_name)
    array_type = data_types.ArrayDataType(tentative_class_name, base_type,
                                          wire_name=wire_name,
                                          parent=parent)
    if schema_id:
      array_type.SetTemplateValue('className', schema_id)
    return array_type

  @property
  def class_name(self):
    return self.values['className']

  @property
  def anonymous(self):
    return 'id' not in self.raw

  @property
  def properties(self):
    return self.values['properties']

  @property
  def isContainerWrapper(self):
    """Is this schema just a simple wrapper around another container.

    A schema is just a wrapper for another datatype if it is an object that
    contains just a single container datatype and (optionally) a kind and
    etag field. This may be used by language generators to create iterators
    directly on the schema. E.g. You could have
        SeriesList ret = api.GetSomeSeriesMethod(args).Execute();
        for (series in ret) { ... }
    rather than
        for (series in ret->items) { ... }

    Returns:
      None or ContainerDataType
    """
    return self._GetPropertyWhichWeWrap() is not None

  @property
  def containerProperty(self):
    """If isContainerWrapper, returns the propery which holds the container."""
    return self._GetPropertyWhichWeWrap()

  def _GetPropertyWhichWeWrap(self):
    """Returns the property which is the type we are wrapping."""
    container_property = None
    for p in self.values['properties']:
      if p.values['wireName'] == 'kind' or p.values['wireName'] == 'etag':
        continue
      if p.data_type.GetTemplateValue('isContainer'):
        if container_property:
          return None
        container_property = p
      else:
        return None
    return container_property

  def __str__(self):
    return '<%s Schema {%s}>' % (self.values['wireName'], self.values)


class Property(template_objects.CodeObject):
  """The definition of a schema property.

  Example property in the discovery schema:
      "id": {"type": "string"}
  """

  def __init__(self, api, schema, name, def_dict, key_for_variants=None):
    """Construct a Property.

    A Property requires several elements in its template value dictionary which
    are set here:
      wireName: the string which labels this Property in the JSON serialization.
      dataType: the DataType of this property.

    Args:
      api: (Api) The Api which owns this Property
      schema: (Schema) the schema this Property is part of
      name: (string) the name for this Property
      def_dict: (dict) the JSON schema dictionary
      key_for_variants: (dict) if given, maps discriminator values to
                        variant schemas.

    Raises:
      ApiException: If we have an array type without object definitions.
    """
    super(Property, self).__init__(def_dict, api, wire_name=name)
    self.ValidateName(name)
    self.schema = schema
    self._key_for_variants = key_for_variants

    # TODO(user): find a better way to mark a schema as an array type
    # so we can display schemas like BlogList in method responses
    try:
      if self.values['wireName'] == 'items' and self.values['type'] == 'array':
        self.schema.values['isList'] = True
    except KeyError:
      pass
    # If the schema value for this property defines a new object directly,
    # rather than refering to another schema, we will have to create a class
    # name for it.   We create a unique name by prepending the schema we are
    # in to the object name.
    tentative_class_name = api.NestedClassNameForProperty(name, schema)
    self._data_type = api.DataTypeFromJson(def_dict, tentative_class_name,
                                           parent=schema, wire_name=name)

  @property
  def code_type(self):
    if self._language_model:
      self._data_type.SetLanguageModel(self._language_model)
    return self._data_type.code_type

  @property
  def safe_code_type(self):
    if self._language_model:
      self._data_type.SetLanguageModel(self._language_model)
    return self._data_type.safe_code_type

  @property
  def primitive_data_type(self):
    if self._language_model:
      self._data_type.SetLanguageModel(self._language_model)
    return self._data_type.primitive_data_type

  @property
  def data_type(self):
    return self._data_type

  @property
  def member_name_is_json_name(self):
    return self.memberName == self.values['wireName']

  @property
  def is_variant_key(self):
    return self._key_for_variants

  @property
  def variant_map(self):
    return self._key_for_variants



#!/usr/bin/python2.7
# Copyright 2012 Google Inc. All Rights Reserved.

"""List of platforms we can target libraries for.

This module defines the canonical names of the platforms that we support for
each language. Those names are used to validate that
"""

__author__ = 'aiuto@google.com (Tony Aiuto)'

ALL = 'all'

# Runtime platforms (per language) that we know about. This list is used to
# validate parameters.
PLATFORMS = {
    ALL: ['*', 'manual'],
    'gwt': ['gwt'],
    # TODO(user): Remove android2 and android3 for 1.12 launch.
    'java': ['android', 'android2', 'android3', 'appengine', 'servlet',
             'generic'],
    'javaproto': ['android', 'appengine', 'servlet', 'generic'],
    }

# Definitions:
# *: required in all environments
# manual: not included by any tools. You must do a manual download to use it.

# Type of files applicable to that platform
FILE_TYPES = {
    ALL: ['binary', 'bundle', 'documentation', 'source', 'license',
          'android-properties', 'proguard-config', 'other']
    }



#!/usr/bin/python2.7
# Copyright 2012 Google Inc. All Rights Reserved.


__author__ = "smulloni@google.com (Jacob Smullyan)"

from google.apputils import basetest
from googleapis.codegen.utilities import json_expander


class JsonExpanderTest(basetest.TestCase):

  def testExpand(self):
    x_val = "foo"
    y_val = "bar"
    d = dict(x=x_val, y=y_val, t1="$x", t2="${y}",
             r={"t": "$x$y", "l": [3, "$x"]})
    expanded = json_expander.ExpandJsonTemplate(d)
    self.assertEquals(x_val, expanded["t1"])
    self.assertEquals(y_val, expanded["t2"])
    self.assertEquals(x_val + y_val, expanded["r"]["t"])
    self.assertEquals(x_val, expanded["r"]["l"][1])

  def testExpandWithAdditionalContext(self):
    y_val = "bar"
    extra = dict(y=y_val)
    d = dict(t="${y}")
    expanded = json_expander.ExpandJsonTemplate(d)
    self.assertNotEquals(y_val, expanded["t"])

    expanded = json_expander.ExpandJsonTemplate(d, extra)
    self.assertEquals(y_val, expanded["t"])

  def testExpandNoSelf(self):
    d = dict(x="aha", t1="$x")
    extra = dict(x="no-no")
    expanded = json_expander.ExpandJsonTemplate(d, extra, use_self=False)
    self.assertEquals("no-no", expanded["t1"])


if __name__ == "__main__":
  basetest.main()



#!/usr/bin/python2.7
# -*- coding: utf-8 -*-
#
# Copyright 2010 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.


"""Tests for name_validator.py."""

__author__ = 'wclarkso@google.com (Will Clarkson)'

from google.apputils import basetest
from googleapis.codegen.utilities import name_validator


class NameValidatorTest(basetest.TestCase):

  def testVariableNameValidator(self):
    good_names = ['$ref', '_a', '_private', 'a_var.name', 't1', 'max-results',
                  'slashes/are/allowed', '/even/at/the/start/and/end/',
                  'now_valid.', '@foo', 'foo$bar']
    bad_names = ['$', '1st_result', '^test', '.variable', '1', '_', 'no spaces',
                 'foo@']

    for varname in good_names:
      name_validator.Validate(varname)
    for varname in bad_names:
      print "'%s'" % varname
      self.assertRaises(name_validator.ValidationError,
                        name_validator.Validate, varname)

  def testApiNameValidator(self):
    good_names = ['valid', 'isValid', 'is2Valid']
    bad_names = ['1noLeadingNumbers', '^test', 'NotValid', 'no-dash',
                 'dot.is.not.valid', 'no spaces', 'no/slash', 'no:colon']

    for varname in good_names:
      name_validator.ValidateApiName(varname)
    for varname in bad_names:
      print "'%s'" % varname
      self.assertRaises(name_validator.ValidationError,
                        name_validator.ValidateApiName, varname)

  def testApiVersionValidator(self):
    good_names = ['v1', 'v1_us', 'v1.2', '1.2', 'with-dash']
    bad_names = ['.1', '1 2', 'no spaces', 'no/slash', 'no:colon']

    for varname in good_names:
      name_validator.ValidateApiVersion(varname)
    for varname in bad_names:
      print "'%s'" % varname
      self.assertRaises(name_validator.ValidationError,
                        name_validator.ValidateApiVersion, varname)

  def testCommentValidator(self):
    good_comments = ['Responses with Content-Type',
                     'application/json',
                     ',',
                     '[minimum: 4.4.1]',
                     'API key. Your API key identifies your project',
                     'OAuth 2.0 token for the current user.']
    # A list of input comments, and their expected replacements
    bad_comments = [('/*', ''),
                    ('*/', ''),
                    ('\"""', ''),
                    ('///', ''),
                    ('\\*', ''),
                    ('/*Some Comment String*/', 'Some Comment String'),
                    ('\""" A long comment string """',
                     ' A long comment string '),
                    ('///Escaped comment string', 'Escaped comment string'),
                   ]

    for comment in good_comments:
      name_validator.ValidateAndSanitizeComment(comment)
    for comment, replacement in bad_comments:
      self.assertEqual(replacement,
                       name_validator.ValidateAndSanitizeComment(comment))

  def testUtf8InComment(self):
    comment = 'Base64-encoded (RFC 4648 §5) data.'
    unicode_comment = comment.decode('utf-8')
    self.assertEqual(unicode_comment,
                     name_validator.ValidateAndSanitizeComment(comment))
    self.assertEqual(
        unicode_comment,
        name_validator.ValidateAndSanitizeComment(unicode_comment))


if __name__ == '__main__':
  basetest.main()



#!/usr/bin/python2.7
# -*- coding: utf-8 -*-
#
# Copyright 2012 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for single_file_library_package."""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

from io import BytesIO

import gflags as flags
from google.apputils import basetest
from googleapis.codegen.filesys import single_file_library_package

FLAGS = flags.FLAGS


class SingleFileLibraryPackageTest(basetest.TestCase):

  def setUp(self):
    self._output_stream = BytesIO()
    self._package = single_file_library_package.SingleFileLibraryPackage(
        self._output_stream)

  def tearDown(self):
    pass

  def testBasicWriteFile(self):
    name1 = 'def'
    content1 = 'contents of def'
    name2 = 'abc'
    content2 = 'contents of abc'
    expected_tmpl = '=== begin: %s\n%s=== end: %s\n'

    stream = self._package.StartFile(name1)
    stream.write(content1)
    stream = self._package.StartFile(name2)
    stream.write(content2)
    self._package.DoneWritingArchive()

    # read it back and verify
    expected = expected_tmpl % (name2, content2, name2)
    expected += expected_tmpl % (name1, content1, name1)
    got = self._output_stream.getvalue()
    self.assertEqual(expected, got)


if __name__ == '__main__':
  basetest.main()



#!/usr/bin/python2.7
# Copyright 2017 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""A LibraryPackage that creates a single output file with delimiters.

Merge all the individual output files into a single stream with delimiters of
the form |=== begin: <path>| and |=== end: <path>|, where the path names are
sorted by name. This is designed to make diffing output against a golden copy
easy, but is is wildly inefficent, as we store all the data before writing any.
"""

from __future__ import absolute_import
from __future__ import division
from __future__ import print_function

import StringIO

from googleapis.codegen.filesys.library_package import LibraryPackage


class SingleFileLibraryPackage(LibraryPackage):
  """The library package."""

  def __init__(self, stream):
    """Create a new SingleFileLibraryPackage.

    Args:
      stream: (file) A file-like object to write to.
    """
    super(SingleFileLibraryPackage, self).__init__()
    self._files = {}
    self._current_file_data = None
    self._final_output_stream = stream

  def StartFile(self, name):
    """Start writing a named file to the package.

    Args:
      name: (str) path which will identify the contents in the archive.

    Returns:
      A file-like object to write the contents to.
    """
    self.EndFile()
    self._current_file_data = StringIO.StringIO()
    self._current_file_name = '%s%s' % (self._file_path_prefix, name)
    return self._current_file_data

  def EndFile(self):
    """Flush the current output file."""
    if self._current_file_data:
      data = self._current_file_data.getvalue()
      self._current_file_data.close()
      self._current_file_data = None
      # File contents may be utf-8
      if isinstance(data, unicode):
        data = data.encode('utf-8')
      # Replace CRLF with LF because in the C# .xml files, some have CRLF but
      # others do not. This causes confusion because depending on how you do
      # a diff on golden output, you get either a change or not.
      self._files[self._current_file_name] = data.replace('\r\n', '\n')

  def DoneWritingArchive(self):
    """Signal that we are done writing the entire package.

    Emit the files.
    """
    self.EndFile()
    for file_name in sorted(self._files):
      print('=== begin: %s' % file_name, file=self._final_output_stream)
      self._final_output_stream.write(self._files[file_name])
      print('=== end: %s' % file_name, file=self._final_output_stream)
    self._final_output_stream.flush()

  def FileExtension(self):
    """Returns the file extension for this archive."""
    return 'txt'

  def MimeType(self):
    """Returns the MIME type for this archive."""
    return 'text/plain'



#!/usr/bin/python2.7

import os
import shutil
import tempfile

from google.apputils import basetest
from googleapis.codegen.filesys import files


class FilesTest(basetest.TestCase):

  def setUp(self):
    self.tempdir = tempfile.mkdtemp()
    for name in 'abc':
      open(os.path.join(self.tempdir, name), 'w').write(name)

  def tearDown(self):
    shutil.rmtree(self.tempdir)

  def testGetFileContentsLocal(self):
    filename = os.path.join(self.tempdir, 'a')
    contents = files.GetFileContents(filename)
    self.assertEquals('a', contents)

  def testIterFilesLocal(self):
    listing = sorted(files.IterFiles(self.tempdir))
    expected = [os.path.join(self.tempdir, x) for x in 'abc']
    self.assertEquals(expected, listing)

  def testIsFileLocal(self):
    self.assertTrue(files.IsFile(os.path.join(self.tempdir, 'a')))
    self.assertFalse(files.IsFile(self.tempdir))

  def testParseGsPath(self):
    path = '/gs/moo-goo-gai-pan/bismarck/marx/leopold.zip'
    spec = files.ParseGsPath(path)
    self.assertIsNotNone(spec)
    expected = ('moo-goo-gai-pan', 'bismarck/marx/leopold.zip')
    self.assertEquals(expected, spec)

if __name__ == '__main__':
  basetest.main()



#!/usr/bin/python2.7
# Copyright 2010 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for generator.py."""

__author__ = 'aiuto@google.com (Tony Aiuto)'

import io
import logging
import os
import zipfile


from google.apputils import basetest

from googleapis.codegen import generator
from googleapis.codegen.filesys import zip_library_package


class GeneratorTest(basetest.TestCase):

  _TEST_DATA_DIR = os.path.join(os.path.dirname(__file__), 'testdata')

  def setUp(self):
    self._output_stream = io.BytesIO()
    self._package = zip_library_package.ZipLibraryPackage(self._output_stream)
    self._path_replacements = {
        '___package_path___': 'pp'
        }

  def VerifyPackageContains(self, file_names, must_not_contain=None):
    """Verify that the output package contains some files.

    Args:
      file_names: List of file names.
      must_not_contain: List of file names which must not be in the package.
    """
    expect_to_see = list(file_names)
    archive = zipfile.ZipFile(io.BytesIO(self._output_stream.getvalue()), 'r')
    info_list = archive.infolist()
    self.assertLess(0, len(info_list))
    for i in info_list:
      path = i.filename
      # Show what we got to help make the test log more useful when we fail.
      logging.info('zip contains: %s', path)
      if path in expect_to_see:
        expect_to_see.remove(path)
      else:
        print 'unexpected file: %s' % path
      if path in (must_not_contain or []):
        self.fail('Found unexpected file %s in archive' % path)
    # We should have seen everything we expect
    self.assertEquals(0, len(expect_to_see))

  def testWalkTemplateTree(self):
    gen = generator.TemplateGenerator()
    gen.SetTemplateDir(os.path.join(self._TEST_DATA_DIR, 'library'))
    gen.WalkTemplateTree(
        'templates', self._path_replacements, {}, {}, self._package)
    self._package.DoneWritingArchive()

    # Now read it back and verify
    self.VerifyPackageContains(['foo', 'bar', 'app.yaml', 'pp/xxx'])

  def testWalkTemplateTreeWithFilter(self):
    gen = generator.TemplateGenerator()
    gen.SetTemplateDir(os.path.join(self._TEST_DATA_DIR, 'library'))
    gen.WalkTemplateTree(
        'templates', self._path_replacements, {}, {}, self._package,
        file_filter=lambda template, output: output != 'bar')
    self._package.DoneWritingArchive()
    self.VerifyPackageContains(['foo'], must_not_contain=['bar'])

  def testWalkTemplateTreeWithFilteredResult(self):
    gen = generator.TemplateGenerator()
    gen.SetTemplateDir(os.path.join(self._TEST_DATA_DIR, 'library'))
    gen.WalkTemplateTree(
        'templates', self._path_replacements, {}, {}, self._package,
        file_filter=lambda template, output: output != 'pp/xxx')
    self._package.DoneWritingArchive()
    self.VerifyPackageContains(['foo', 'bar'], must_not_contain=['pp/xxx'])

  def testWalkTemplateTreeWithFilteredTemplate(self):
    gen = generator.TemplateGenerator()
    gen.SetTemplateDir(os.path.join(self._TEST_DATA_DIR, 'library'))
    gen.WalkTemplateTree(
        'templates', self._path_replacements, {}, {}, self._package,
        file_filter=lambda template, output: template != 'bar.tmpl')
    self._package.DoneWritingArchive()
    self.VerifyPackageContains(['foo'], must_not_contain=['bar'])



if __name__ == '__main__':
  basetest.main()



#!/usr/bin/python2.7
#
# Copyright 2011 Google Inc. All Rights Reserved.

"""HTMLStripper based on HTMLParser."""


__author__ = 'wclarkso@google.com (Will Clarkson)'

import HTMLParser


class HTMLStripper(HTMLParser.HTMLParser):
  """Simple class to strip tags from HTML."""

  def __init__(self):
    HTMLParser.HTMLParser.__init__(self)
    self.reset()
    self.fed = []

  def handle_data(self, d):
    self.fed.append(d)

  def GetFedData(self):
    return ''.join(self.fed)

  def ClearFedData(self):
    self.fed = []



#!/usr/bin/python2.7
# -*- coding: utf-8 -*-
#
# Copyright 2010 Google Inc. All Rights Reserved.
#
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

"""Tests for utilities.py."""

__author__ = 'aiuto@google.com (Tony Aiuto)'

from google.apputils import basetest
import googleapis.codegen.utilities as utilities


class UtilitiesTest(basetest.TestCase):

  def testCamelCase(self):
    """Basic CamelCase functionality."""
    self.assertEquals('HelloWorld', utilities.CamelCase('hello_world'))
    self.assertEquals('HelloWorld', utilities.CamelCase('hello-world'))
    self.assertEquals('HelloWorld', utilities.CamelCase('helloWorld'))
    self.assertEquals('HelloWorld', utilities.CamelCase('Hello_world'))
    self.assertEquals('HelloWorld', utilities.CamelCase('_hello_world'))
    self.assertEquals('HelloWorld', utilities.CamelCase('helloWorld'))
    self.assertEquals('HelloWorld', utilities.CamelCase('hello.world'))
    self.assertEquals('HELLOWORLD', utilities.CamelCase('HELLO_WORLD'))
    self.assertEquals('HelloWorld', utilities.CamelCase('hello/world'))
    self.assertEquals('HelloWorld', utilities.CamelCase('/hello/world/'))
    self.assertEquals('', utilities.CamelCase(''))
    self.assertEquals(' ', utilities.CamelCase(' '))
    self.assertEquals(' ', utilities.CamelCase('. '))

  def testUnCamelCase(self):
    """Basic CamelCase functionality."""
    # standard case
    self.assertEquals('hello_world', utilities.UnCamelCase('helloWorld'))
    self.assertEquals('hello_world', utilities.UnCamelCase('Hello_world'))
    self.assertEquals('hello_world', utilities.UnCamelCase('helloWorld'))
    self.assertEquals('hello_world', utilities.UnCamelCase('HELLO_WORLD'))
    self.assertEquals('hello_world', utilities.UnCamelCase('HELLOworld'))
    self.assertEquals('hello_world', utilities.UnCamelCase('helloWORLD'))
    self.assertEquals('hello2_world', utilities.UnCamelCase('Hello2World'))

    # keep existing separators
    self.assertEquals('hello_world', utilities.UnCamelCase('hello_world'))
    self.assertEquals('_hello_world', utilities.UnCamelCase('_hello_world'))
    self.assertEquals('_hello_world', utilities.UnCamelCase('_HelloWorld'))
    self.assertEquals('hello__world', utilities.UnCamelCase('Hello__World'))

    # embedded acronym
    self.assertEquals('hello_xw_orld', utilities.UnCamelCase('HelloXWorld'))

    # minimal input
    self.assertEquals('h', utilities.UnCamelCase('H'))
    self.assertEquals('', utilities.UnCamelCase(''))

    # Other cases involving expanded alphabet.
    self.assertEquals('_', utilities.UnCamelCase('_'))
    self.assertEquals('hello-world', utilities.UnCamelCase('hello-world'))
    self.assertEquals('hello.world', utilities.UnCamelCase('hello.world'))
    self.assertEquals('hello/world', utilities.UnCamelCase('hello/world'))
    self.assertEquals('hello world', utilities.UnCamelCase('Hello World'))
    self.assertEquals(' ', utilities.UnCamelCase(' '))

  def testSanitizeDomain(self):
    self.assertIsNone(utilities.SanitizeDomain(None))
    self.assertEquals('google.com', utilities.SanitizeDomain('google.com'))
    self.assertEquals('google.com', utilities.SanitizeDomain('GooglE.com'))
    self.assertEquals('google.com', utilities.SanitizeDomain('goo|gle.com'))
    self.assertEquals('google.com', utilities.SanitizeDomain('goo gle.com'))
    self.assertEquals('googl.com', utilities.SanitizeDomain('googlê.com'))
    self.assertEquals('www_test.appspot.com',
                      utilities.SanitizeDomain('www-test.appspot.com'))

  def testReversedDomainComponents(self):
    self.assertEquals([],
                      utilities.ReversedDomainComponents(''))
    self.assertEquals(['com', 'google'],
                      utilities.ReversedDomainComponents('google.com'))

  def testNoSpaces(self):
    self.assertIsNone(utilities.NoSpaces(None))
    self.assertEquals('', utilities.NoSpaces(''))
    self.assertEquals('', utilities.NoSpaces(' '))
    self.assertEquals('abc', utilities.NoSpaces('a b  c '))

if __name__ == '__main__':
  basetest.main()
